<?php
/**
 * Simple Forge configuration settings class, used for general configuration
 *
 * @package core
 */
class ForgeConfig {
	/** 
	 * Configuration variables specific to Hippo namespace or the application
	 * using Hippo library
	 */
	private static $config_vars= array();
	
	/**
	 * Retrieve a configuration variable by a key
	 *
	 * @return 	matched configuration variable or FALSE
	 * @param		key		Key to search configuration variables for
	 */
	public static function get($key) {
		if (isset(self::$config_vars[$key]))
			return self::$config_vars[$key];
		return FALSE;
	}

	/**
	 * Sets a single or multiple configuration variable
	 *
	 * Takes either two parameters representing a key and value
	 * combination, or an associative array of configuration 
	 * variables
	 *
	 * @return	void
	 */
	public static function set() {
		$args= func_get_args();
		$num_args= func_num_args();
		if ($num_args == 2) {
			$key= $args[0];
			$value= $args[1];
			self::$config_vars[$key]= $value;
		}
		elseif ($num_args == 1) {
			$variables= $args[0];
			self::$config_vars= self::$config_vars + $variables;
		}
	}
}

/**
 * Main controller for forge web requests.  This class simply
 * acts as a traffic cop, pushing out the requests to the 
 * appropriate module handlers which do the real work
 *
 * @package core
 */
class ForgeController {
	/** Base URL */
	public $base_url= null;
	/** Request that was issued from the web server or CLI */
	public $stub= null;
	/** The matched ForgeUrlRule instance for the request */
	public $matched_rule= null;
	/** Enabled module instances */
	public $modules= array();
	/** Name of active module, if any */
	public $active_module= null;
	/** Template engine for use in displaying stuff */
	public $handler= null;
	/** Cache object */
	public $cache= null;
	/** Session object */
	public $session= null;
	/** Is a user logged in? */
	public $user_logged_in= FALSE;

	/**
	 * Constructor for the controller
	 */
	public function __construct() {
		/* Begin output buffering and cleaning of inputs */
		HippoHttpResponse::init();

		/* Add listeners here, before anything gets going... */
		HippoEvents::listen('on_before_display_template', array($this, 'add_menu_items'));

		/* The session holds all user-session related variables */
		$this->session= new ForgeCookieSession();
		$this->user_logged_in= (bool) $this->session->get('user_id');

		/* 
		 * Initialize a cache engine for lookup-type information
		 * needed on page requests
		$this->cache= HippoCache::engine('memcached');
		$this->cache->init(array('servers'=>AppConfig::get_config('MEMCACHED_SERVERS')))
		 */
		$this->cache= HippoCache::engine('file');

		/* Load the activated modules into the internal array */
		$this->modules= ForgeModules::init_enabled($this);

		/* Grab the base URL from the Site class */
		$this->base_url= ForgeConfig::get('DIR_HTTP');

		/* Start with the entire URL coming from web server... */
		$start_url= (isset($_SERVER['REQUEST_URI'])
					? $_SERVER['REQUEST_URI']
					: $_SERVER['SCRIPT_NAME'] .
						(isset($_SERVER['PATH_INFO'])
						? $_SERVER['PATH_INFO']
						: '') .
							((isset($_SERVER['QUERY_STRING']) && ($_SERVER['QUERY_STRING'] != ''))
							? '?' . $_SERVER['QUERY_STRING']
							: ''));

		/* Strip out the base URL from the requested URL */
		/* but only if the base URL isn't / */
		if ('/' != $this->base_url) {
			$start_url= str_replace($this->base_url, '', $start_url);
		}

		/* Trim off any leading or trailing slashes */
		$start_url= trim($start_url, '/');

		/* Normalize for idiocy */
		$start_url= str_replace("\\", "/", $start_url);

		/* Remove the querystring from the URL */
		if (strpos($start_url, '?') !== FALSE ) {
			list($start_url, $query_string)= explode('?', $start_url);
		}

		/* Return $_GET values to their proper place */
		if( !empty($query_string) ) {
			parse_str($query_string, $_GET);
		}

		$this->stub= $start_url;

		$this->init_core_url_rules();
	}

	/**
	 * Simply creates and registers some core URL rules
	 * that should be accounted for
	 */
	private function init_core_url_rules() {
		$rules= array();
		
		/* Setup fullsite search rules */
		$rule= new ForgeUrlRule("search");
		$rule->set_inbound_match("search")
			->set_inbound_callback(array($this, 'search'));
		$rules[$rule->key]= $rule;
		
		/* Setup voting rules */
		$rule= new ForgeUrlRule("vote");
		$rule->set_inbound_match("/vote\/([[:alpha:]]+)\/(\d+)\/*$/i")
			->set_inbound_match_vars(array('item_type','item_id'))
			->set_inbound_callback(array($this, 'vote'));
		$rules[$rule->key]= $rule;
		
		/* Setup tagging rules */
		$rule= new ForgeUrlRule("tag");
		$rule->set_inbound_match("/tag\/([[:alpha:]]+)\/(\d+)\/*$/i")
			->set_inbound_match_vars(array('item_type','item_id'))
			->set_inbound_callback(array($this, 'tag'));
		$rules[$rule->key]= $rule;
	}

	/**
	 * Packages the request and sends it to the
	 * appropriate handler to complete the work
	 */
	public function dispatch_request() {
		
		HippoEvents::notify('on_before_dispatch_request', $this);
		$stub= $this->stub;

		/* Handle homepage requests first, then start the match process */
		if ($stub == '')
			$this->do_home();

		if (FALSE === ($matched_rule= ForgeUrl::match_rule_to_inbound_stub($stub)))
			$this->do_404();

		/* 
		 * OK, at this point, we've got a matched rule.
		 * The rule will specify whether a UI handler is needed,
		 * what callback to execute, if any, and will
		 * contain matched regex variables, if any, that the 
		 * rule culled from the stub
		 */
		$this->matched_rule= $matched_rule;

		if ($matched_rule->requires_ui)
			$this->init_ui();

		if ($stub == '') 
			$this->handler->display('dashboard');
		else {
			if (!is_null($this->handler)) {
				$in_match_vars= $matched_rule->get_inbound_match_vars();
				if (! empty($in_match_vars))
					$this->handler->handler_vars= $this->handler->handler_vars + $in_match_vars;
			}

			/* 
			 * At this point, we'll have a stub request, 
			 * which we then parse and pass off to either a
			 * module to fulfill the request, or another 
			 * core handler...
			 *
			 * For module requests, the name of the module
			 * will ALWAYS be the first piece of the stub.
			 */
			$stub_pieces= explode('/', $stub);

			if (in_array($stub_pieces[0], array_keys($this->modules))) {
				$module= array_shift($stub_pieces);
				$this->active_module= $module;
				$matched_rule->exec_callback();
			}
		}	
		HippoEvents::notify('on_after_dispatch_request', $this);
	}

	/**
	 * Handle homepage requests 
	 */
	private function do_home() {
		$this->init_ui();
		$this->handler->display('dashboard');
	}

	/**
	 * Handle unknown requests
	 */
	private function do_404() {
		/* Clear headers and fire off handler for the 404 page */
		ob_clean();
		header("Status: 404 Not Found");
		$this->init_ui();
		$this->handler->display('404');
	}

	/**
	 * Sets up the templating system for use by handlers in
	 * displaying templatized contents.
	 *
	 * @return	void
	 */
	private function init_ui() {
		$this->handler= new ForgeUIHandler($this);

		$this->handler->assign('HTTP_ROOT', ForgeConfig::get('HTTP_ROOT'));
		$this->handler->assign('HTTPS_ROOT', ForgeConfig::get('HTTPS_ROOT'));
		$this->handler->assign('LOGGED_IN', (bool) $this->session->get('user_id'));
		$this->handler->assign('IS_ADMIN',  (bool) $this->session->get('is_admin'));

		ForgeUI::init($this->handler);
	}

	/**
	 * Adds menu items for display in the layout template
	 */
	public function add_menu_items() {

		$main_menu_items= array();
		$main_menu_items[]= array(
			'title'=>ForgeTranslate::get('home')
			, 'link'=>ForgeConfig::get('HTTP_ROOT')
		);
		if (is_null($this->active_module))
			$main_menu_items[0]['active']= TRUE;
		$this->handler->append('menu_items', $main_menu_items);
	}
}

/**
 * Class for creating and returning URLs translated for
 * the forge framework's rewrite rules for pretty URLs
 *
 * @package core
 */
class ForgeUrl {
	/** Cache of URL rule map */
	private static $url_rules= array();

	/**
	 * Returns a URL for outputting to the application
	 *
	 * @return	string	URL for outputting
	 * @param		key			string or array to match against
	 * @param		params	(optional) array of GET params to append to URL
	 */
	public static function out($key, $params= array()) {
		if (isset(self::$url_rules[$key])) {
			$rule= self::$url_rules[$key];
			$url= rtrim(ForgeConfig::get('HTTP_ROOT'), '/') . '/';
			if (! is_null($rule->out_expression)) {
				/* Replace outbound expression params with params supplied */
				$out_expression= $rule->out_expression;
				if (! empty($params)) {
					foreach ($params as $key=>$value) {
						$out_expression= str_replace('{$' . $key . '}', $value, $out_expression);
						unset($params[$key]);
					}
				}
				$url.= $out_expression;
			}
			else {
				/* OK, no outbound expression replacement... */
				$url.= $rule->get_outbound();
			}
			if (!empty($params)) {
				if (is_array($params))
					$url.= '?' . http_build_query($params);
				elseif (is_string($params))
					$url.= '?' . ltrim($params, '?');
			}
			return $url;
		}
		return rtrim(ForgeConfig::get('HTTP_ROOT'), '/') . '/';
	}

	/**
	 * Adds an array of URL map rules
	 *
	 * @return	void
	 * @param		rules		Array of URL map rules
	 */
	public static function append_rules($rules= array()) {
		self::$url_rules= self::$url_rules + (array) $rules;
	}

	/** 
	 * Matches an incoming stub to the set of known URL rules
	 *
	 * @return	FALSE or an instance of ForgeUrlRule
	 * @param		stub		The request stub
	 */
	public static function match_rule_to_inbound_stub($stub) {
		foreach (self::$url_rules as $rule)
			if ($rule->match_inbound($stub))
				return $rule;
		return FALSE;
	}


	/**
	 * Utility method for inspecting the cached
	 * URL rules...
	 *
	 * @return	array of URL rules
	 */
	public static function get_rules() {
		return self::$url_rules;
	}
}

/**
 * Class providing ability for providing URL classes
 * that can be matched based on a regular expression
 *
 * @package	core
 */
class ForgeUrlRule {
	/** ID of the rule */
	public $key= null;
	/** Expression to match an incoming stub */
	protected $in_match= array();
	/** Callback to run on match */
	protected $in_callback= null;
	/** Rule variable assignments for incoming match */
	protected $in_match_vars= array();
	/** Expression to match an URL to be written to the response stream */
	public $out_expression= null;
	/** Rule variable assignments for outgoing match */
	protected $out_match_vars= array();
	/** Does the callback need a UI handler? (DEFAULT == TRUE) */
	public $requires_ui= TRUE;

	/**
	 * Public constructor
	 *
	 * @param	key		A name or identifier for the URL rule
	 */
	public function __construct($key) {
		$this->key= $key;
	}
	
	/**
	 * Sets the callback method to call on a match, and returns
	 * the object instance for chaining purposes
	 *
	 * @return	this
	 * @param		callback	Method or function name to call on matching
	 */
	public function set_inbound_callback($callback) {
		$this->in_callback= $callback;
		return $this;
	}
	
	/**
	 * Sets the match string for inbound request stubs
	 * and returns the object itself for chaining purposes
	 *
	 * @return	this
	 * @param		match_string	Regex to match a stub against
	 */
	public function set_inbound_match($match_string) {
		$this->in_match[]= $match_string;
		return $this;
	}
	
	/**
	 * Sets the inbound variable match replacements and returns 
	 * the object itself for chaining purposes
	 *
	 * @return	this
	 * @param		variables			Variable names to assign match pieces
	 */
	public function set_inbound_match_vars($variables) {
		$variables= (array) $variables;
		foreach ($variables as $key)
			$this->in_match_vars[$key]= null;
		return $this;
	}

	/**
	 * Returns the collection on inbound matched variables
	 *
	 * @return	array of variables or null
	 */
	public function get_inbound_match_vars() {
		return $this->in_match_vars;
	}

	/**
	 * Returns the outbound expression to use.  If an outbound
	 * expression has not been set, it returns the first inbound match
	 * string by default.  This means less code for easy URLs...
	 *
	 * @return	string	URL outbound expression
	 */
	public function get_outbound() {
		if (! empty($this->out_expression))
			return $this->out_expression;
		else
			return $this->in_match[0];
	}
	
	/**
	 * Sets the match string for outbound request stubs
	 * and returns the object itself for chaining purposes
	 *
	 * @return	this
	 * @param		match_string	Regex to match a stub agaist
	 */
	public function set_outbound_expression($match_string) {
		$this->out_expression= $match_string;
		return $this;
	}
	
	/**
	 * Sets the outbound variable match replacements and returns 
	 * the object itself for chainting purposes
	 *
	 * @return	this
	 * @param		variables			Variable names to assign match pieces
	 */
	public function set_outbound_match_vars($variables) {
		$variables= (array) $variables;
		foreach ($variables as $key)
			$this->out_match_vars[$key]= null;
		return $this;
	}

	/**
	 * Matches a stub to the rewrite rule.
	 *
	 * @return	TRUE or FALSE if the rule matches the stub
	 * @param		stub		Stub to match against
	 */
	public function match_inbound($stub) {
		foreach ($this->in_match as $match_string) {
			if ($match_string{0} != '/') {
				/* Straight match, no regex */
				if ($stub == $match_string)
					return TRUE;
				else
					return FALSE;
			}
			else {
				/* A regex rule */
				if (preg_match($match_string, $stub, $matches) == 1) {
					array_shift($matches); /* Pop the full matched string */
					$num_matches= count($matches);
					if ($num_matches > 0) {
						$variable_keys= array_keys($this->in_match_vars);
						for ($i=0; $i<$num_matches;++$i)
							$this->in_match_vars[$variable_keys[$i]]= $matches[$i];
					}
					return TRUE;
				}
				return FALSE;
			}
		}
	}

	/**
	 * Executes the inbound callback
	 *
	 * @return	result of callback
	 */
	public function exec_callback() {
		if (is_array($this->in_callback)) {
			$object= $this->in_callback[0];
			$method= $this->in_callback[1];
			return $object->$method();
		}
		else {
			$function= $this->in_callback;
			return $function();
		}
	}
}

/**
 * Simple modules class for the forge
 *
 * @package core
 */
class ForgeModules {

	const CACHE_MODS_ENABLED_KEY= 'modules_enabled';
	const CACHE_MODS_ENABLED_INCLUDE_KEY= 'modules_enabled_include';
	const CACHE_MODS_AVAILABLE_KEY= 'modules_available';

	private static $loaded_modules= array();

	/**
   * Grabds enable modules and runs init() on each
	 * 
 	 * @param		cache		null or a HippoActsAsCacheEngine object
	 * @return	TRUE or FALSE if load succeeded
   */
	public static function init_enabled($controller) {

		$dir= APP_DIR . '/modules/enabled';
		$cache= $controller->cache;

		if (! is_dir($dir))
			return FALSE;

		$modules= array();
		if (!is_null($cache) && $cache instanceof HippoActsAsCacheEngine) {
			/**
			 * To prevent excessive re-caching, what we do is set no expiration
			 * on the cache, and do a quick filetime check on the directory against
			 * the cached file.  If the directory has changed, we purge the cache
			 * and repopulate.  This allows administrators to activate
			 * and deactivate modules at will without having to worry about cache
			 * invalidation...
			 */
			$dir_modified= filemtime($dir);
			$cache_modified= $cache->get_modified_time(ForgeModules::CACHE_MODS_ENABLED_KEY);

			if ($cache_modified
				&& ($dir_modified > $cache_modified)) {
				$cache->delete(ForgeModules::CACHE_MODS_ENABLED_KEY);
			}
			else {
				$cache_options= array('seconds_to_expire'=>0, 'is_object'=>TRUE);
				if (FALSE !== ($modules= $cache->get(ForgeModules::CACHE_MODS_ENABLED_KEY, $cache_options))) {
					if (ForgeModules::init_modules($modules, $controller))
						return self::$loaded_modules;
				}
			}
		}
		$lambda= create_function('$file','return ($file{0} != ".");'); /* Strips out ., .. and hiddens */
		$modules= array_values(array_filter(scandir($dir), $lambda));
		
		if (!empty($modules) && !is_null($cache) && $cache instanceof HippoActsAsCacheEngine) {
			$cache->set(ForgeModules::CACHE_MODS_ENABLED_KEY, $modules);
		}
		if (ForgeModules::init_modules($modules, $controller))
			return self::$loaded_modules;
 	}

	/**
	 * Checks to see if module file has been loaded
	 * by verifying against the static laoded_modules
	 * array.  This is necessary because class_exists()
	 * unfortunately fires __autoload(), which isn't 
	 * aware of module file locations and will error out...
	 *
	 * @return	TRUE or FALSE
	 * @param		module		Name of module to check for
	 */
	public static function is_loaded($module) {
		return isset(self::$loaded_modules[$module]);
	}

	/**
	 * Loads an array of modules and returns whether
	 * or not the entire array was loaded and subsequently
	 * init()ed
	 *
	 * @return	array of loaded and inited modules
	 * @param		modules		Array of module names to load
	 */
	public static function init_modules($modules, $controller) {
		$result= TRUE;

		foreach ($modules as $module)
			$result&= ForgeModules::load_module($module, $controller);
		if ($result)
			foreach (self::$loaded_modules as $module)
				if (method_exists($module, 'init'))
					$module->init();
		return $result;
	}				

	/**
	 * Includes into the execution path the module
	 * file for the module
	 *
	 * @return	void
	 * @param		module	Name of the module to include
	 */
	public static function load_module($module, $controller) {
		if (ForgeModules::is_loaded($module))
			return TRUE;

		$module_name= $module . 'module';
		$module_file= APP_DIR . "/modules/enabled/{$module}/module.php";
		if (file_exists($module_file))
			include($module_file);
		self::$loaded_modules[$module]= new $module_name($controller);
		return TRUE;
	}
}

/**
 * Singular module class from which to inherit
 *
 * @package core
 */
class ForgeModule {
	/** The instance of a forge controller directing traffic */
	protected $controller= null;

	/**
	 * Map of URL rules to catch for this module...
	 */
	private $url_map= array();

	/**
	 * Map of UI switches to show for each action of this module...
	 */
	public $ui_map= array();
	
	/**
	 * Base constructor.  Sets controller instance
	 *
	 * @param	controller	Instance of a forge controller
	 */
	public function __construct($controller) {
		$this->controller= $controller;
	}
}

/**
 * Class providing i10n and l10n capabilities
 *
 * @package core
 */
class ForgeTranslate {
	/** Static cache of translated key/value pairs for the current locale */
	private static $cached_translations= array();
	/** Domain is already set? */
	private static $domain_is_set= FALSE;

	/**
	 * Returns a plurarlized and translated string from the key
	 *
	 * @return 	string						Translated string, if found, or the string itself
	 * @param		singular_key			Key for the translation to lookup for singular tense
	 * @param		pluralized_key		Key for the pluralized tense
	 * @param		num_things				The number of things being referred to..
	 * @param		params						(optional) Array of parameters to replace placeholders in format string, if present
	 */
	public static function nget($singular_key, $pluralized_key, $num_things, $params= array()) {
		if (isset(self::$cached_translations[$singular_key.$num_things]))
			$translated= self::$cached_translations[$singular_key.$num_things];
		else {
			if (! self::$domain_is_set)
				ForgeTranslate::set_domain();
			$translated= ngettext($singular_key, $pluralized_key, $num_things);
			self::$cached_translations[$singular_key.$num_things]= $translated;
		}
		$params= (array) $params;
		if (!empty($params))
			return vsprintf($translated, $params);
		else
			return $translated;
	}

	/**
	 * Returns a translated string from the key
	 *
	 * @return 	string		Translated string, if found, or the string itself
	 * @param		key				Key for the translation to lookup
	 * @param		params	  (optional) Array of parameters to replace placeholders in format string, if present
	 */
	public static function get($key, $params= array()) {
		if (isset(self::$cached_translations[$key]))
			$translated= self::$cached_translations[$key];
		else {
			if (! self::$domain_is_set)
				ForgeTranslate::set_domain();
			$translated= gettext($key);
			self::$cached_translations[$key]= $translated;
		}
		$params= (array) $params;
		if (!empty($params))
			return vsprintf($translated, $params);
		else
			return $translated;
	}

	/**
	 * Sets the domain for translation lookups
	 *
	 * @return	void
	 */
	public static function set_domain() {
		bindtextdomain('mysqlforge', ForgeConfig::get('DIR_LC_MESSAGES'));
		textdomain('mysqlforge');
		self::$domain_is_set= TRUE;
	}
}

/**
 * Child class of themed UI controller
 *
 * @package	core
 */
class ForgeUIHandler extends HippoThemedUIHandler {
	/** Subdirectory for templates */
	public $template_subdir= '';
	/** Internal errors collection, written to session on response */
	public $errors= array();
	/** Controller instance */
	private $controller= null;

  /**
	 * Constructor creates the session, initializes the default theme
	 *
	 * @param	controller	Instance of a ForgeController
   */
	public function __construct($controller) {
		$this->controller= $controller;

		HippoEvents::listen('on_before_display_template', array($this, 'on_before_display_template'));
		HippoEvents::listen('on_before_httpresponse_redirect', array($this, 'on_before_httpresponse_redirect'));

		parent::__construct();
  }

	/**
	 * Writes session upon redirect, first appending the errors
	 * collection to the session.
	 *
	 * @return void
	 */
	public function on_before_httpresponse_redirect() {
		if (!empty($this->errors))
			$this->controller->session->set('errors', $this->errors);
		$this->controller->session->write();
	}

	/**
	 * Writes session upon template display, first removing the errors
	 * collection from the session.
	 *
	 * @return void
	 */
	public function on_before_display_template() {
		$this->controller->session->delete('errors');
		$this->controller->session->delete('message');
		$this->controller->session->write();
	}

	/**
	 * Simple assignment of useful variables into the template engine, then
	 * call parent's method to complete auto-assignment of handler_vars
	 *
	 * @param	template_name	Name of the template to display
	 */
	public function display($template_name) {
   	$this->init_theme_from_dir(APP_DIR . '/templates/');

		/* Automatically include session information in the templates... */
    $this->assign('session', $this->controller->session->get_all());

		/* Add error handling into templates */
		if (FALSE !== $this->controller->session->get('errors'))
			$this->errors+= $this->controller->session->get('errors');
		$this->assign('errors', $this->errors);

		/* 
		 * Automatic messaging across requests.  Simply do:
		 * $this->set_message("blah"); 
		 * anywhere in handler to have message appear in 
		 * templates (for a single request) 
		 */
		if (FALSE !== $this->controller->session->get('message'))
			$this->assign('message', $this->controller->session->get('message'));

		/*
		 * We escape everything going to the template!
		 */
		/* Turn off the darn iconv notices about incomplete multi-byte characters...
		$original_reporting= error_reporting();
		error_reporting(0);
		$this->handler_vars= $this->recurse_escape_output($this->handler_vars);
		error_reporting($original_reporting);
		 */
		if ($this->template_subdir != '')
			$this->template_subdir= rtrim($this->template_subdir, "/") . '/';

		$template_munged_name= str_replace('/','_',$template_name);
		HippoEvents::notify('on_before_display_template');
		HippoEvents::notify("on_before_display_$template_munged_name");
		parent::display($this->template_subdir . $template_name);
		HippoEvents::notify("on_after_display_$template_munged_name");
		HippoEvents::notify('on_after_display_template');
		exit;
	}

	/**
	 * Recursively escape the handler's variables for outputting into the template
	 *
	 * @return	array		Array of escaped stuff
	 * @param		mixed		Either an array or not.  Recurse if array, otherwise escape and return
	 */
	private function recurse_escape_output($subject) {
		if (is_object($subject)) 
			return $subject;
		if (! is_array($subject))
			return iconv('UTF-8','UTF-8//IGNORE',$subject);
		$results= array();
		foreach ($subject as $key=>$value)
			$results[$key]= $this->recurse_escape_output($value);
		return $results;
	}

	/**
	 * Adds an error to the internal error collection which is 
	 * automatically exported to the template upon display and then
	 * destroyed (single-session only) 
	 *
	 * @return	void
	 * @param		key				Key of the error (match with HTML element ID...)
	 * @param		message		Message to store
	 */
	public function add_error($key, $message) {
		$this->errors[$key]= (string) $message;
	}

	/**
	 * Sets a message which will be saved, for a single page refresh, in
	 * a special session variable and exported automatically into the 
	 * template system.  Useful to send the user a save successful message.
	 *
	 * @return	void
	 * @param		message		Message to store
	 */
	public function set_message($message) {
		$this->controller->session->set('message', (string) $message);
	}

	/**
	 * Assigns a variable to the handler's template variables, which, upon
	 * display of the template itself, are extracted into the template
	 *
	 * @param	key		Key to assign variable
	 * @param	value	Value to assign variable
	 */
	public function assign($key, $value) {
		$this->handler_vars[$key]= $value;
	}

	/**
	 * Appends a variable to the handler's template variables, which, upon
	 * display of the template itself, are extracted into the template.
	 *
	 * @note	This is only different from assign() in that it does not override
	 * 				existing set variables for this key
	 *
	 * @param	key		Key to append variable
	 * @param	value	Value to append to variable
	 */
	public function append($key, $value) {
		if (! isset($this->handler_vars[$key]))
			$this->assign($key,$value);
		elseif (is_array($value))
			$this->handler_vars[$key]= array_merge($this->handler_vars[$key], $value);
		else {
			if (! is_array($this->handler_vars[$key]))
				$this->handler_vars[$key]= array($this->handler_vars[$key]);
			$this->handler_vars[$key][]= $value;
		}
	}

	/**
	 * Utility method for determining if a key is set
	 * in the handler's variables...
	 *
	 * @return	true or false
	 * @param		key		Key to check
	 */
	public function exists($key) {
		return isset($this->handler_vars[$key]);
	}
}

/**
 * Helper class for some of repetitive UI actions, 
 * including error output, HTML select dropdowns, etc.
 *
 * @package	core
 */
class ForgeUI {
	/** A handler instance to use for reference */
	private static $handler= null;
	/**
	 * Initialize the UI with a handler (saves us doing
	 * a bunch of $GLOBALS hacks...
	 *
	 * @return	void
	 * @param		handler		A ForgeUIHandler instance
	 */
	public static function init($handler) {
		self::$handler= $handler;
	}

	/**
	 * Includes a template and extracts supplied variables into
	 * that template.
	 *
	 * @return	void
	 * @param		template	Name of the template 
	 * @param		args			Array of variables to extract into template
	 */
	public static function include_template($template, $args= array()) {
		if (file_exists($template)) {
			extract(self::$handler->handler_vars);
			if (! empty($args) && is_array($args))
				extract($args);
			include($template);
		}
	}
	/**
	 * Outputs HTML options tag with options
	 * to the output buffer
	 *
	 * @return void
	 * @param		selected		Array index that should be marked as selected (optional)
	 * @param		css_class		optional CSS class name to output (optional)
	 */
	public static function html_options($options, $selected= null) {
		$options= (array) $options;
		$options_html= '';
		foreach ($options as $key=>$value) {
			$options_html.= "<option value=\"{$key}\"";
			if ($key == $selected)
				$options_html.= " selected=\"true\"";
			$options_html.=" />" . htmlentities($value) . "</option>";
		}
		echo $options_html;
	}

	/**
	 * Outputs an error container.  Takes an ID of the element
	 * to check for an error and outputs an error box if found...
	 *
	 * @return	void
	 * @param		key			Element to lookup in the errors collection
	 */
	public static function error($key) {
		$errors= $GLOBALS['controller']->handler->errors; /** @TODO: get rid of this hideous GLOBALS...*/
		if (isset($errors[$key]))
			print <<<ENDOFHTML
 <div class="error">
  {$errors[$key]}
 </div>
ENDOFHTML;
	}
}

/** @package	core functions */

/**
 * Takes a string a converts it to a timestamp that 
 * can be used by strftime() and locales...
 *
 * @see smarty implementation in libs/plugins/shared.make_timestamp.php
 * @return	timestamp
 * @param		from			String to convert from
 */
function forge_str2timestamp($from) {
	if(empty($from))
		$time= time();
	elseif (preg_match('/^\d{14}$/', $from))
		/*  It is mysql timestamp format of YYYYMMDDHHMMSS */
		$time= mktime(substr($from, 8, 2),substr($from, 10, 2),substr($from, 12, 2),
									 substr($from, 4, 2),substr($from, 6, 2),substr($from, 0, 4));
	elseif (is_numeric($from))
		/* It is a numeric from, assume it is already a timestamp */
		$time= (int) $from;
	else {
		/* strtotime should handle it */
		$time= strtotime($from);
		if ($time == -1 || $time === FALSE)
			$time= time();
	}
	return $time;
}

/**
 * Utility function to make tedious isset($thing) ? $thing : '' checks
 * easier
 *
 * @return	either first or second parameter, depending on isset() of the first
 * @param		thing_array		Array of things to check
 * @param		key						Index/key to check
 * @param		alternative 	Alternative if no set (optional)
 */
function forge_key_issetor($thing_array, $key, $alternative= FALSE) {
	if (isset($thing_array[$key]))
		return $thing_array[$key];
	else
		return $alternative;
}

/**
 * Utility function to make tedious isset($thing) ? $thing : '' checks
 * easier
 *
 * @return	either first or second parameter, depending on isset() of the first
 * @param		thing					Thing to check for
 * @param		alternative 	Alternative if no set (optional)
 */
function forge_issetor($thing, $alternative= FALSE) {
	if (isset(${$thing}))
		return $thing;
	else
		return $alternative;
}
?>
