<?php
/**
 * The CommentHandler is the handler which handles all actions
 * having to do with commenting.  Adding, editing, deleting comments, etc.
 */
class CommentHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= '';
		parent::__construct();

		/* All commenting functionality requires user to be logged in */
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');

		HippoEvents::listen('on_after_project_add_comment', array($this, 'on_after_project_add_comment'));
		HippoEvents::listen('on_after_worklog_add_comment', array($this, 'on_after_worklog_add_comment'));
		HippoEvents::listen('on_after_tool_add_comment', array($this, 'on_after_tool_add_comment'));
	}

	/**
	 * Callback for when a new comment is added to a 
	 * project.
	 *
	 * Notify via email all interested parties (either from the 
	 * remote notifications list or the commenting parties for 
	 * this project)
	 *
	 * @return 	bool	Successful or not?
	 */
	public function on_after_project_add_comment() {
		$project_id= $this->handler_vars['item_id'];
		/* Grab a list of email addresses to notify of feedback */
		$sql=<<<ENDOFSQL
SELECT fu.email
FROM ForgeUser fu
INNER JOIN ProjectObservers o 
ON fu.user_id = o.user_id
WHERE o.project_id = ?
AND o.notify_on_feedback = 1
AND o.user_id != ?
ENDOFSQL;
		if ($connection= HippoDb::master()) {
			$connection->add_param($project_id);
			$connection->add_param($this->session->get('user_id'));
			if (! $emails= $connection->get_column($sql))
				$emails= array();

			$sql=<<<ENDOFSQL
SELECT fu.email
FROM ForgeUser fu
INNER JOIN ProjectComment c
ON fu.user_id = c.added_by
WHERE c.added_by != ?
GROUP BY c.added_by
ENDOFSQL;
			$connection->add_param($this->session->get('user_id'));
			if ($more_emails= $connection->get_column($sql))
				$emails= array_unique(array_merge($emails, $more_emails));

			if (!empty($emails)) {
				/* Grab the name of the project for the subject line */
				$reader= new ProjectReader($connection);
				$reader->project_id= (int) $project_id;
				if (!$project= $reader->get_one($sql))
					return FALSE;

				/* Construct email and send to observing parties */
				include_once(APP_DIR . '/opt/swift/lib/Swift.php');
				include_once(APP_DIR . '/opt/swift/lib/Swift/Connection/Sendmail.php');
				$mailer= new Swift(new Swift_Connection_Sendmail()); /** @see docs on SwiftMailer.org */
				$message_subject= "New feedback from " . $this->session->get('display_name') . " on project: " . $project['name'];
				$admin_email= "admin@forge.mysql.com";

				/* Initialize the templating engine */
				$message_template= 'email/project-add-comment';
				$template_engine= new HippoSmartyTemplateEngine();
				$template_engine->template_extension= ".tpl"; /* This theme uses templates ending in .tpl */
				$template_engine->set_compiled_template_dir(AppConfig::get_config('SMARTY_ENGINE_COMPILED_TEMPLATE_DIR'));
				$template_engine->set_plugins_dir(AppConfig::get_config('SMARTY_ENGINE_PLUGINS_DIR'));
				$template_engine->set_compile_check(AppConfig::get_config('SMARTY_ENGINE_COMPILE_CHECK'));
				$template_engine->set_force_compile(AppConfig::get_config('SMARTY_ENGINE_FORCE_COMPILE'));
				$template_engine->set_template_dir(APP_DIR . '/templates');
				$template_engine->assign('project_id', $project_id);
				$template_engine->assign('project_name', $project['name']);
				$template_engine->assign('added_by', $this->session->get('display_name'));
				$template_engine->assign('comment_text', $this->handler_vars['comment_text']);
				$message_text= $template_engine->fetch($message_template);

				$message= new Swift_Message($message_subject, $message_text);
				/* Email each observer */
				foreach ($emails as $observer_email)
					$mailer->send($message, $observer_email, $admin_email);

				$this->session->set('message', $this->session->get('message') . "<br/>" . count($emails) . " Observers notified.");
			}
		}
	}

	/**
	 * Callback for when a new comment is added to a worklog
	 * task.
	 *
	 * Notify via email all interested parties (either from the 
	 * remote notifications list or the commenting parties for 
	 * this task)
	 *
	 * @return 	bool	Successful or not?
	 */
	public function on_after_worklog_add_comment() {
		$task_id= $this->handler_vars['item_id'];
		/* Grab a list of email addresses to notify of feedback */
		$sql=<<<ENDOFSQL
SELECT email
FROM user u 
INNER JOIN observers o 
ON u.id = o.dev_id
WHERE o.task_id = ?
ENDOFSQL;
		if ($connection= HippoDb::master(array('db'=>'worklog_remote'))) {
			$connection->add_param($task_id);
			if (! $emails= $connection->get_column($sql))
				$emails= array();
	
			/* Grab the list of commenter's emails for this task */
			if ($connection= HippoDb::master(array('db'=>'forge'))) {
				$sql=<<<ENDOFSQL
SELECT fu.email
FROM ForgeUser fu
INNER JOIN WLTaskComment c
ON fu.user_id = c.added_by
WHERE c.added_by != ?
GROUP BY c.added_by
ENDOFSQL;
				$connection->add_param($this->session->get('user_id'));
				if ($more_emails= $connection->get_column($sql))
					$emails= array_unique(array_merge($emails, $more_emails));

				if (!empty($emails)) {
					/* Construct email and send to observing parties */
					include_once(APP_DIR . '/opt/swift/lib/Swift.php');
					include_once(APP_DIR . '/opt/swift/lib/Swift/Connection/Sendmail.php');
					$mailer= new Swift(new Swift_Connection_Sendmail()); /** @see docs on SwiftMailer.org */
					$message_subject= "WL#{$task_id}: New feedback from " . $this->session->get('display_name');
					$admin_email= "worklog@forge.mysql.com";

					/* Initialize the templating engine */
					$message_template= 'email/worklog-add-comment';
					$template_engine= new HippoSmartyTemplateEngine();
					$template_engine->template_extension= ".tpl"; /* This theme uses templates ending in .tpl */
					$template_engine->set_compiled_template_dir(AppConfig::get_config('SMARTY_ENGINE_COMPILED_TEMPLATE_DIR'));
					$template_engine->set_plugins_dir(AppConfig::get_config('SMARTY_ENGINE_PLUGINS_DIR'));
					$template_engine->set_compile_check(AppConfig::get_config('SMARTY_ENGINE_COMPILE_CHECK'));
					$template_engine->set_force_compile(AppConfig::get_config('SMARTY_ENGINE_FORCE_COMPILE'));
					$template_engine->set_template_dir(APP_DIR . '/templates');
					$template_engine->assign('task_id', $task_id);
					$template_engine->assign('added_by', $this->session->get('display_name'));
					$template_engine->assign('comment_text', $this->handler_vars['comment_text']);
					$message_text= $template_engine->fetch($message_template);

					$message= new Swift_Message($message_subject, $message_text);
					/* Email each observer */
					foreach ($emails as $observer_email)
						$mailer->send($message, $observer_email, $admin_email);

				$this->session->set('message', $this->session->get('message') . "<br/>" . count($emails) . " Observers notified.");
				}
			}
		}
	}

	/**
	 * Callback for when a new comment is added to a 
	 * code snippet or other tool
	 *
	 * Notify via email all interested parties (either from the 
	 * remote notifications list or the commenting parties for 
	 * this task)
	 *
	 * @return 	bool	Successful or not?
	 */
	public function on_after_tool_add_comment() {
		/** @todo What observers?! */
		return true;
		$tool_id= $this->handler_vars['item_id'];
		/* Grab the list of commenter's emails for this tool */
		if ($connection= HippoDb::master()) {
			$sql=<<<ENDOFSQL
SELECT fu.email
FROM ForgeUser fu
INNER JOIN ToolComment c
ON fu.user_id = c.added_by
WHERE c.added_by != ?
AND tc.tool_id = ?
GROUP BY c.added_by
ENDOFSQL;
			$connection->add_param($this->session->get('user_id'));
			$connection->add_param($this->handler_vars['tool_id']);
			if ($more_emails= $connection->get_column($sql))
				$emails= array_unique(array_merge($emails, $more_emails));

			if (!empty($emails)) {
				/* Construct email and send to observing parties */
				include_once(APP_DIR . '/opt/swift/lib/Swift.php');
				include_once(APP_DIR . '/opt/swift/lib/Swift/Connection/Sendmail.php');
				$mailer= new Swift(new Swift_Connection_Sendmail()); /** @see docs on SwiftMailer.org */
				$message_subject= "WL#{$this->handler_vars['tool_id']}: New feedback from " . $this->session->display_name;
				$admin_email= "worklog@forge.mysql.com";

				/* Initialize the templating engine */
				$message_template= 'email/tool-add-comment';
				$template_engine= new HippoSmartyTemplateEngine();
				$template_engine->template_extension= ".tpl"; /* This theme uses templates ending in .tpl */
				$template_engine->set_compiled_template_dir(AppConfig::get_config('SMARTY_ENGINE_COMPILED_TEMPLATE_DIR'));
				$template_engine->set_plugins_dir(AppConfig::get_config('SMARTY_ENGINE_PLUGINS_DIR'));
				$template_engine->set_compile_check(AppConfig::get_config('SMARTY_ENGINE_COMPILE_CHECK'));
				$template_engine->set_force_compile(AppConfig::get_config('SMARTY_ENGINE_FORCE_COMPILE'));
				$template_engine->set_template_dir(APP_DIR . '/templates');
				$template_engine->assign('task_id', $this->handler_vars['task_id']);
				$template_engine->assign('added_by', $this->session->get('display_name'));
				$template_engine->assign('comment_text', $this->handler_vars['comment_text']);
				$message_text= $template_engine->fetch($message_template);

				$message= new Swift_Message($message_subject, $message_text);
				/* Email each observer */
				foreach ($emails as $observer_email)
					$mailer->send($message, $observer_email, $admin_email);

				$this->session->set('message', $this->session->get('message') . "<br/>" . count($emails) . " Observers notified.");
			}
		}
	}

	/**
	 * Adds a new comment to some item, notifies any listening events
	 * for the addition of the comment, and then redirects back to the appropriate
	 * view page
	 *
	 * @return	void
	 */
	public function add_comment() {
		$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3);
		if (isset($this->handler_vars['item'])) {
			$item= $this->handler_vars['item'];
			$item_type= $item_type_map[$item];
		}
		else
			$this->errors[]= 'No item specified';

		if (isset($this->handler_vars['item_id']))
			$item_id= (int) $this->handler_vars['item_id'];
		else
			$this->errors[]= 'No ID specified';

		$comment_text= null;
		if (isset($this->handler_vars['comment_text']))
			$comment_text= $this->handler_vars['comment_text'];

		if (empty($this->errors)) {
			switch (strtolower($item)) {
			case 'worklog':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "worklog/task.php?id={$item_id}";
				break;
			case 'project':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$item_id}";
				break;
			case 'tool':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "tools/tool.php?id={$item_id}";
				break;
			case 'doc':
				/* Hmmmm, what to do here?...*/
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "/project.php?id={$project_id}";
				break;
			case 'forge_user':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project_id}";
				break;
			}

			$reader= new CommentReader(HippoDb::master());
			$reader->item_type= (int) $item_type;
			$reader->item_id= (int) $item_id;
			$new_comment_id= (int) $reader->get_count() + 1;

			$comment= new Comment(HippoDb::master());
			$comment->item_type= (int) $item_type;
			$comment->item_id= (int) $item_id;
			$comment->comment_id= $new_comment_id;
			$comment->added_on= date('Y-m-d H:i:s');
			$comment->added_by= $this->session->get('user_id');
			$comment->original_text= $comment_text;
			$comment->clean_text= TextDecorator::clean_and_codify($comment_text);

			if ($comment->save()) {
				//HippoEvents::notify("on_after_{strtolower($item)}_add_comment", $comment);
			}
			else {
				$errors= $comment->get_errors();
				$this->errors['comment_text']= "Error saving comment: " . $errors['comment_text'];
			}
			HippoHttpResponse::redirect($url_redirect);
		}
		else
			print_r($this->errors); //HippoHttpResponse::redirect($SERVER['HTTP_REFERER']);
	}
}
?>

