<?php
/* Check for existence of FEEDGENERATOR_DIR */
if (!ForgeConfig::get('FEEDGENERATOR_DIR')) {
	trigger_error('FEEDGENERATOR_DIR not defined.', E_USER_ERROR);
	exit;
}
include_once(ForgeConfig::get('FEEDGENERATOR_DIR') . 'FeedGenerator.php');

/**
 * The FeedHandler is the handler which handles all actions
 * related to retrieving and caching RSS and ATOM feeds
 */
class FeedHandler {
	/** Internal handler variables.  Exported to template on display */
	public $handler_vars= array();
	/** Cache engine for lookup information */
	protected $cache= null;
	/** Search keyword */
	protected $keyword= '';
	/** Feed protocol (RSS / Atom) */
	protected $protocol= 'RSS 2.0';
	/** Feed content type (projects, code_snippets, worklog_tasks, keyword) */
	protected $type= 'wl';
	/** Feed content type (Projects, Code Snippets, Worklog Tasks, Keyword) */
	protected $type_clean;
	
	/** Time to expire cached feeds */
	const CACHE_EXPIRATION_IN_SECONDS= 10; /* 20 minutes */

	public function __construct() {
		foreach ($_GET as $key=>$value)
			$this->handler_vars[$key]= $value;
		foreach ($_POST as $key=>$value)
			$this->handler_vars[$key]= $value;

		/* 
		 * Initialize a cache engine for lookup-type information
		 * needed on page requests
		 */
		$this->cache= HippoCache::engine('file');
		$this->cache_dir= AppConfig::get_config('SMARTY_ENGINE_CACHE_DIR');
		$this->cache_options= array('seconds_to_expire'=>FeedHandler::CACHE_EXPIRATION_IN_SECONDS
																, 'is_object'=>FALSE);

		$this->handler_vars['DIR_HTTP']= AppConfig::get_config('DIR_HTTP');
		$this->handler_vars['DIR_HTTPS']= AppConfig::get_config('DIR_HTTPS');
	}

	/**
	 * Grabs newest stuff from forge and returns as an associative
	 * array of items (to then be placed into templates)
	 *
	 * @return	array	Array of newest X things
	 * @param		num_items		(optional) Number of items to return (defaults to 20)
	 */
	protected function get_recent_items($num_items= 20) {
		
		switch ($this->type) {
			case 'code_snippets':  // Code Snippets
				$reader= new ToolReader(HippoDb::slave());
				/* Override default SELECT as we need specific info for the feed */
				$reader->select_sql =<<<ENDOFSQL
				SELECT
					t.tool_id as item_id
					, UNIX_TIMESTAMP(t.added_on) as added_on
					, fu.display_name as added_by_display
					, t.title as title
					, cs.clean_notes as description
					, pl.name as program_language
					, '' as version
				FROM Tools t
					INNER JOIN ForgeUser fu
						ON t.added_by = fu.user_id
					INNER JOIN CodeSnippets cs
						ON t.tool_id = cs.tool_id
					INNER JOIN ProgramLanguage pl
						ON cs.program_language = pl.program_language_id
ENDOFSQL;
				$reader->tool_type= 1; /* Code Snippet == 1 */
				$reader->order('added_on', 'desc');
				if (! $newest_snippets= $reader->get($num_items))
					$newest_snippets= array();

				return $newest_snippets;
				break;
			case 'projects':  // Projects
				$reader= new ProjectReader(HippoDb::slave());
				/* Override default SELECT as we need specific info for the feed */
				$reader->select_sql =<<<ENDOFSQL
				SELECT
					p.project_id as item_id
					, UNIX_TIMESTAMP(p.added_on) as added_on
					, fu.display_name as added_by_display
					, p.name as title
					, pt.description as description
					, '' as program_language
					, '' as version
				FROM Project p
					INNER JOIN ProjectText pt
						ON p.project_id = pt.project
					INNER JOIN ForgeUser fu
						ON p.added_by = fu.user_id
ENDOFSQL;
				$reader->order('added_on', 'desc');
				if (! $newest_projects= $reader->get($num_items))
					$newest_projects= array();
					
				return $newest_projects;
				break;
			case 'workload_tasks':  // Workload tasks			
				$reader= new WLTaskReader(HippoDb::slave());
				/* Override default SELECT as we need specific info for the feed */
				$reader->select_sql =<<<ENDOFSQL
				SELECT
					t.id as item_id
					, UNIX_TIMESTAMP(t.lastmodified) as added_on
					, 'MySQL AB' as added_by_display
					, t.title as title
					, t.description as description
					, '' as program_language
					, t.version as version
				FROM WLTask t
ENDOFSQL;
				$reader->order('lastmodified', 'desc');
				if (! $newest_tasks= $reader->get($num_items))
					$newest_tasks= array();
					
				return $newest_tasks;
				break;
				
			default: // Wrong type
				return null;
		}
	}

	/**
	 * Grab an RSS feed of the most recent "stuff"
	 */
	public function display_feed() {
		$cache_key= str_replace(array(' ', '.'), '_', $this->protocol).'_'.$this->type.'.xml';
		
		header('Content-Type: application/xhtml+xml');
		
		HippoEvents::notify('on_before_display_feed');
		
		if ($cached_data= $this->cache->get($cache_key, $this->cache_options)) {		
			echo $cached_data;
			exit;
		}
		else {
			//Creating an instance of FeedWriter class. 
			$feed = new FeedGenerator($this->protocol);
			
			//Setting the channel elements
			switch ($this->type) {
				case 'code_snippets':  // Code Snippets
					$feed_link= 'tools/';
					$item_link= AppConfig::get_config('DIR_HTTP') . 'tools/tool.php?id=';
					$title= 'CODE EXAMPLE: ';
					break;
				case 'projects':  // Projects
					$feed_link= 'projects/';
					$item_link= AppConfig::get_config('DIR_HTTP') . 'projects/project.php?id=';
					$title= 'PROJECT: ';
					break;
				case 'workload_tasks':  // Workload tasks
					$feed_link= 'worklog/';
					$item_link= AppConfig::get_config('DIR_HTTP') . 'worklog/task.php?id=';
					$title= 'WORKLOAD TASK: ';	
					break;
				default:
					$feed_link= '';
					$item_link= '';
					$title= '';
					break;
			}
			
			//Use wrapper functions for common channel elements
			$feed->setTitle('MySQL Forge Newest '.$this->type_clean);
			$feed->setLink(AppConfig::get_config('DIR_HTTP') . $feed_link);
			$feed->setDescription('This is a feed containing new '.strtolower($this->type_clean).' added recently to MySQL Forge');
			
			if ($this->protocol == ATOM) {
				$feed->setChannelElement('updated', date(DATE_ATOM, time()));				
			}
			if ($this->protocol == RSS2) {
				$feed->setChannelElement('language', 'en-us');
				$feed->setChannelElement('pubDate', date(DATE_RSS, time()));
			}			
						
			//Get feed items									
			if (is_array($this->get_recent_items())) {
				foreach ($this->get_recent_items() as $item) {
					//Create an empty FeedItem
					$feed_item= $feed->createNewItem();
				
					$description= '';
					if ($this->type == 'code_snippets') {
						$description.= "<p>Program Language <b>{$item['program_language']}</b></p>";
						//$code_text= htmlspecialchars($item['code_text']);
						//$description.= "<p>Code:</p> <pre>{$code_text}</pre>";
					}
					$description.= $item['description'];
					
					$feed_item->setDescription($description);
					$feed_item->setDate((int) $item['added_on']);
					$feed_item->setLink($item_link . $item['item_id']);
					$feed_item->setTitle($title . $item['title']);
					
					if ($this->protocol == ATOM) {
						$feed_item->addElement('author', array('name'=>$item['added_by_display']));
					}
					
					if ($this->protocol == RSS2) {
						$feed_item->addElement('guid', $item_link . $item['item_id'], array('isPermaLink'=>'true'));
					}
					
					$feed->addItem($feed_item);
				}
			}
			
			/* Create the cached feed */
			$uncached_data= $feed->generateFeed();
			$this->cache->set($cache_key, $uncached_data);

			/* Display feed */		
			echo $uncached_data;
 		}
 		
		HippoEvents::notify('on_after_display_feed');
	}

	public function set_keyword($keyword) {
		$this->keyword= $keyword;
	}

	public function set_protocol($protocol) {
		$this->protocol= $protocol;
	}
	
	public function set_type($type) {
		$this->type= $type;
		if ($type == 'keyword') {
			$this->type_clean= 'Search Results for "'.strip_tags($this->keyword).'"';
		}
		else {
			$this->type_clean= ucwords(str_replace('_', ' ', $type));
		}
	}
}

