<?php
/**
 * The TagHandler is the handler which handles all actions
 * having to do with tags.  Adding, editing, merging, etc.
 */
class TagHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= '';
		$this->handler_vars['CUR_NAV']= 'home';
		parent::__construct();
		/* All tagging functionality requires user to be logged in */
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');
	}

	/**
	 * Processes requests for tag additions, deletions.
	 *
	 * @see templates/add-tag.tpl
	 *
	 * @return	void
	 */
	public function process_tags() {
		$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3, 'person'=>4, 'forge_user'=>4);
		if (isset($this->handler_vars['item'])) {
			$item= $this->handler_vars['item'];
			$item_type= $item_type_map[$item];
		}
		else
			$this->errors[]= 'No item specified';

		if (isset($this->handler_vars['id']))
			$item_id= (int) $this->handler_vars['id'];
		else
			$this->errors[]= 'No ID specified';

		$tag_text= null;
		if (isset($this->handler_vars['tag_text']))
			$tag_text= $this->handler_vars['tag_text'];
		else
			$this->errors[]= 'No tag text specified';

		if (empty($this->errors)) {

			/*
			 * Split the tags via comma-delimiters, then remove
			 * any special characters and normalize
			 */
			$tags= explode(',', $tag_text);
			$tags= array_map('trim', $tags);
			$tags= array_unique($tags);
			$num_new_tags= count($tags);

			/* Strip out the crap */
			$tmp_tags = array();
			for ($i=0;$i<$num_new_tags;++$i) {
				if ($tags[$i] == '')
					continue;
				$in= preg_replace("/[^a-zA-Z0-9_\ ]/",'', $tags[$i]);
				while ($in !== str_replace('  ',' ', $in))
					$in= str_replace('  ',' ', $in); /* remove any double-triple-etc spaces */
				$tmp_tags[]= $in;
			}
			$tags= $tmp_tags;
			$num_review_tags= 0; /* See below, these are partial matched tags */
			unset($tmp_tags);
                
			/*
			 * OK, first we check for the tag IDs of exact matching
			 * tags to the array of added tags, excluding already
			 * tagged items.
			 */
			if (! empty($tags)) {
				$connection= HippoDb::master();
				$tags_in= implode("\",\"", $tags);
				$sql=<<<ENDOFSQL
SELECT SQL_NO_CACHE t.tag_id, t.tag_text 
FROM Tag t
LEFT JOIN ItemTags it
ON t.tag_id = it.tag_id
AND it.item_id = {$item_id} 
AND it.item_type = {$item_type}
WHERE it.item_id IS NULL 
AND t.tag_text IN ("{$tags_in}")
ENDOFSQL;
				if ($exact_tag_records= $connection->get_rows($sql)) {
					/* A container for tag phrases to remove from our collection */
					$tags_to_remove= array();
					/* Let's just add the exact ones right away */
					foreach ($exact_tag_records as $exact_tag_record) {
						$exact_tags[]= $exact_tag_record['tag_id'];
						$tags_to_remove[]= $exact_tag_record['tag_text'];
					}
					if ($this->update_existing_tags($exact_tags, $item_type, $item_id)) {
						/* 
						 * Remove the exact matches that we've already attached to
						 * the item.
						 */
						$tmp_tags= array();
						foreach ($tags as $tag)
							if (! in_array($tag, $tags_to_remove))
								$tmp_tags[]= $tag;
						$tags= $tmp_tags;
						unset($tmp_tags);
					}
				}
			}

			if (empty($this->errors)) {
				/* 
				 * OK, now we try to match close tags to see if the 
				 * user mistyped or would like to use existing tags
				 */
				$review_tags= array(); /* Keyed based on original tag text */
				$new_tags= array(); /* Holds brand new tags, with no similar or exact matching */
				$num_new_tags = count($tags);
				for ($i=0;$i<$num_new_tags;++$i) {
					$current_tag_text= $tags[$i];
					/* Only bother with words more than 2 characters in length */
					if (strlen($current_tag_text) < 3) {
							$new_tags[]= $current_tag_text;
							continue;
					}

					$sql= "SELECT tag_id, tag_text FROM Tag WHERE tag_text LIKE ? AND tag_text != ?";
					$connection->add_param("%$current_tag_text%");
					$connection->add_param($current_tag_text);
					if ($matched_tag_records= $connection->get_rows($sql)) 
						$review_tags[$current_tag_text]= $matched_tag_records;
					else
						$new_tags[]= $current_tag_text;
				}
			
				$num_new_tags= count($new_tags);
				$num_review_tags= count($review_tags);
			
				/**
				 * OK, now let's add the non-matching new tags
				 */
				if ($num_new_tags > 0) {
					foreach ($new_tags as $new_tag) {
						if (!$this->add_new_tag($new_tag, $item_type, $item_id))
							break;
					}
				}
			}

			/* Now we remove any tags in our deleted_tags collection */
			if (empty($this->errors)) {
				if (isset($this->handler_vars['deleted_tags'])
					&& ! empty($this->handler_vars['deleted_tags']))
						$this->delete_existing_tags(array_values($this->handler_vars['deleted_tags']), $item_type, $item_id);
			}
			
			if ($num_review_tags > 0) {
				$this->handler_vars['review_tags']= $review_tags;
				$this->handler_vars['item']= ucfirst($item);
				$this->handler_vars['item_id']= $item_id;
				$this->handler_vars['redirect']= $_SERVER['HTTP_REFERER'];
				$this->handler_vars['page_title']= 'Review Tags';
				$this->display('review-tags');
				exit;
			}
			else {
				if (empty($this->errors))
					$this->session->set('message', "Save successful.");

				/* We return to referring page */
				HippoHttpResponse::redirect($_SERVER['HTTP_REFERER']);
			}
			/* We return to referring page */
			HippoHttpResponse::redirect($_SERVER['HTTP_REFERER']);
		}
	}

	/**
	 * Add new or existing tags from the review tags interface
	 *
	 * @return void
	 */
	public function review_tags() {
		$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3, 'person'=>4, 'forge_user'=>4);
		if (isset($this->handler_vars['item'])) {
			$item= $this->handler_vars['item'];
			$item_type= $item_type_map[$item];
		}
		else
			$this->errors[]= 'No item specified';

		if (isset($this->handler_vars['id']))
			$item_id= (int) $this->handler_vars['id'];
		else
			$this->errors[]= 'No ID specified';

		$new_tags= array();
		if (isset($this->handler_vars['new_tags']))
			$new_tags= array_values($this->handler_vars['new_tags']);

		$review_tags= array();
		if (isset($this->handler_vars['review_tags']))
			$review_tags= array_values($this->handler_vars['review_tags']);
			
		if (! empty($new_tags))
			foreach ($new_tags as $tag)
				$this->add_new_tag($tag, $item_type, $item_id);

		if (! empty($review_tags))
			$this->update_existing_tags($review_tags, $item_type, $item_id);

		$this->session->set('message', "Save successful");
		HippoHttpResponse::redirect($this->handler_vars['redirect']);

	}
	
	/**
	 * Adds a brand new tag
	 *
	 * @return	bool			Insert successful?
	 * @param		tag_text	Tag phrase
	 * @param		item_type	The type of item being tagged
	 * @param		item_id		The ID of the item being tagged
	 */
	private function add_new_tag($tag_text, $item_type, $item_id) {
		/* Insert into the main tag table first */
		$connection= HippoDb::master();
		$sql= "INSERT INTO Tag (tag_id, tag_text, num_tags) VALUES (NULL, ?, 1)";
		$connection->add_param($tag_text);
		$connection->start_transaction();
		if (! $connection->query($sql)) {
			$this->errors['tag']= "Error occurred processing tags: " . $connection->get_last_error_message();
			$connection->rollback();
			return FALSE;
		}
		else {
			/* Insert into the relation table */
			$sql= "INSERT INTO ItemTags (item_type, item_id, tag_id) VALUES ({$item_type},{$item_id}," . $connection->last_insert_id() . ")";
			if (! $connection->query($sql)) {
				$this->errors['tag']= "Error occurred processing tags: " . $connection->get_last_error_message();
				$connection->rollback();
				return FALSE;
			}
			else {
				$connection->commit();
				return TRUE;
			}
		}
	}

	/**
	 * Updates an existing tag
	 *
	 * @return	bool			Did the update succeed?
	 * @param		tag_ids		Array of IDs of tags to update
	 * @param		item_type	The type of item being tagged
	 * @param		item_id		The ID of the item being tagged
	 */
	private function update_existing_tags($tag_ids, $item_type, $item_id) {
		$connection= HippoDb::master();
		/* Let's just add the exact ones right away */
		$sql= "INSERT INTO ItemTags (item_type, item_id, tag_id) VALUES ";
		foreach ($tag_ids as $tag)
			$sql.= "({$item_type},{$item_id},{$tag}),";

		$sql= rtrim($sql, ',');
		$connection->start_transaction();
		if (! $connection->query($sql)) {
			$this->errors['tag']= "Error occurred processing tags: " . $connection->get_last_error_message();
			$connection->rollback();
			return FALSE;
		}
		else {
			/* 
			 * We need to update the num_tags counter 
			 * of the Tag table now 
			 */
			$sql= "UPDATE Tag SET num_tags=num_tags+1 WHERE tag_id IN (" . implode(',', $tag_ids) . ")";
			if (! $connection->query($sql)) {
				$this->errors['tag']= "Error occurred processing tags: " . $connection->get_last_error_message();
				$connection->rollback();
				return FALSE;
			}
			else {
				$connection->commit();
				return TRUE;
			}
		}
	}

	/**
	 * Removes existing tags and deletes from main Tag table if necessary
	 *
	 * @return	bool		Did the delete succeed?
	 * @param		tag_ids		Array of IDs of tags to update
	 * @param		item_type	The type of item being tagged
	 * @param		item_id		The ID of the item being tagged
	 */
	private function delete_existing_tags($tag_ids, $item_type, $item_id) {
		$connection= HippoDb::master();
		/* Let's just add the exact ones right away */
		$sql= "DELETE FROM ItemTags WHERE tag_id IN(" . implode(',', $tag_ids) . ")	AND item_type = ? AND item_id = ? ";
		$connection->add_param($item_type);
		$connection->add_param($item_id);
		$connection->start_transaction();
		if (! $connection->query($sql)) {
			$this->errors['tag']= "Error occurred deleting tags: " . $connection->get_last_error_message();
			$connection->rollback();
			return FALSE;
		}
		else {
			/* 
			 * We need to update the num_tags counter 
			 * of the Tag table now 
			 */
			$sql= "UPDATE Tag SET num_tags=num_tags-1 WHERE tag_id IN (" . implode(',', $tag_ids) . ")";
			if (! $connection->query($sql)) {
				$this->errors['tag']= "Error occurred processing tags: " . $connection->get_last_error_message();
				$connection->rollback();
				return FALSE;
			}
			else {
				/* 
				 * Remove any tags with no more relations...
				 */
				$sql= "DELETE FROM Tag WHERE num_tags=0";
				if (! $connection->query($sql)) {
					$this->errors['tag']= "Error occurred deleting tags: " . $connection->get_last_error_message();
					$connection->rollback();
					return FALSE;
				}
				else {
					$connection->commit();
					return TRUE;
				}
			}
		}
	}
}
?>
