<?php
/**  
 * TextDecorator
 * 
 * Contains various methods to decorate plain text, either
 * by rewriting strings to URLs, codifying text, etc.
 */
class TextDecorator {
	/**
	 * Converts a given text by adding URLs for keywords
	 * of worklog tasks and bugs
	 * 
	 * @return  string
	 * @param   subject    String to convert
	 */
	public static function convert_bugs_and_worklogs($subject) {
		/* Convert BUG#NNN and WL#NNN to URLs... */
		$regex_matches= array(
			'/BUG\s*\#\s*(\d+)/i'
			, '/WL\s*\#\s*(\d+)/i');
		/* Use back references to grab the ID numbers from the matcher */
		$regex_replaces= array(
			'<a href="http://bugs.mysql.com/bug.php?id=\\1">BUG#\\1</a>'
			, '<a href="http://forge.mysql.com/worklog/task.php?id=\\1">WL#\\1</a>');

			return preg_replace($regex_matches, $regex_replaces, $subject);
	}  

	/**
	 * Codifies text using the GeSHi library
	 * 
	 * @return  string
	 * @param   subject   		String to convert
	 */
	public static function codify($subject) {
		$code_regex= '/\[code\s*(lang|language)\=[\"\'](\w+)[\"\']\]([\D\S]+?)\[\/code\]/';
		$code_matches= array();
		/* Format the code with GeSHi */
		include_once(APP_DIR . '/opt/geshi/geshi.php');
		while (preg_match($code_regex, $subject, $code_matches) == 1) {
			$language= trim(strtolower($code_matches[2])); // 0-index is the full match
			$code_sample= $code_matches[3];
			$code_sample= str_replace("\t", "  ", $code_sample); /* Replace tabs with spaces */
			$geshi= new GeSHi($code_sample, $language);
			$geshi->set_header_type(GESHI_HEADER_PRE);
			$geshi->enable_classes();
			$geshi->enable_line_numbers(GESHI_NORMAL_LINE_NUMBERS);
			$code_formatted= $geshi->parse_code();
			$subject= preg_replace($code_regex, $code_formatted, $subject);
			$code_matches= array(); //reset
		}
		return $subject;
	}

	/**
	 * Highlights the text as code in the supplied language
	 *
	 * @return	string		The marked up code
	 * @param		subject		The text to markup
	 * @param		language	The language to use for highlighting
	 */
	public static function syntax_highlight($subject, $language) {
		/* Format the code with GeSHi */
		include_once(APP_DIR . '/opt/geshi/geshi.php');
		$geshi= new GeSHi($subject, $language);
		$geshi->enable_classes();
		$geshi->enable_line_numbers(GESHI_NORMAL_LINE_NUMBERS);
		return $geshi->parse_code();
	}

	/**
	 * Returns a cleaned and syntax-highlighted string of HTML
	 *
	 * @return	string		Cleaned and codified text
	 * @param		subject		The text to cut into code pieces
	 */
	public static function clean_and_codify($subject) {
		$original= $subject;
		$code_pieces= array();
		$code_regex= '/[\[\<]code\s*(lang|language)\=[\"\'](\w+)[\"\'][\]\>]([\D\S]+?)[\[\<]\/code[\]\>]/';
		$code_delimiter= "CODECODECODE";

		/* First split the text into code and non-code blocks */
		while (preg_match($code_regex, $subject, $code_matches) == 1) {
			$language= trim(strtolower($code_matches[2])); // 0-index is the full match
			$code_sample= $code_matches[3];
			$entire_code_string= $code_matches[0];
			$code_sample= str_replace("\t", "  ", $code_sample); /* Replace tabs with spaces */
			$code_pieces[]= array('lang'=>$language
													, 'text'=>$code_sample);
			$subject= str_replace($entire_code_string, $code_delimiter, $subject);
			$code_matches= array(); //reset
		}

		/* 
		 * Next, do the same thing with markup sections
		 * We use HTMLPurifier here for safe checks with some allowed 
		 * tags for ease of use
		 */
		$subject= TextCleaner::clean_html($subject);
		
		/* 
		 * Now $subject should contain CleanMarkup\n|||CODE|||\nCleanMarkup...
		 * We now replace the code sections by passing an executable string
		 * to the regex parser (the /e option) and using the syntax_highlight
		 * function to do the grunt work
		 */
		$num_code_pieces= count($code_pieces);
		$i= 0;
		if ($num_code_pieces > 0) {
			$replacement= "TextDecorator::syntax_highlight(trim(\$code_pieces[\$i]['text'], \"\r\n \"), \$code_pieces[\$i++]['lang']);";
			$subject= preg_replace('/' . $code_delimiter . '/e', $replacement, $subject);
		}
		return $subject;
	}
}
?>
