<?php
/**
 * The ToolHandler is the handler which handles all actions
 * for the displaying the public code snippet information and comments
 * and other developer and DBA tools
 */
class ToolHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= 'tools';
		$this->handler_vars['CUR_NAV']= 'tools';
		parent::__construct();
	}

	/**
	 * Displays a single tool.
	 *
	 * This method essentially is a switch statement to 
	 * display a customized view for the different tool types 
	 * @return void
	 */
	public function display_tool() {
		$tool_id= (int) $this->handler_vars['id'];
		$reader= new ToolReader(HippoDb::slave());
		/* Override the default SELECT to just get the tool type */
		$reader->select_sql= "SELECT tool_type FROM Tools";
		$reader->tool_id= $tool_id;
		if (! $tool_type= $reader->get_one())
			HippoHttpResponse::redirect(AppConfig::get_config("DIR_HTTP"));

		$tool_type= (int) $tool_type['tool_type'];
		switch ($tool_type) {
		case 1: /* A code snippet */
			$this->display_code_snippet();
			break;
		}
	}

	/**
	 * Removes a tool and all it's associated relationships
	 *
	 * @note Only an admin is allowed to execute this...
	 */
	public function delete_tool() {
		if (FALSE !== $this->session->get('is_admin')) {
			if (! isset($this->handler_vars['id']))
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
			$tool_id= $this->handler_vars['id'];
			$cnn= HippoDb::master();
			$cnn->start_transaction();

			$sql_deletes= array(
				"DELETE FROM ItemTags WHERE item_type = 2 AND item_id = ?"
			, "DELETE FROM Observers WHERE item_type = 2 AND item_id = ?"
			, "DELETE FROM VoteSummary WHERE item_type = 2 AND item_id = ?"
			, "DELETE FROM Votes WHERE item_type = 2 AND item_id = ?"
			, "DELETE FROM Comments WHERE item_type = 2 AND item_id = ?"
			, "DELETE FROM Snippet2SnippetCategory WHERE snippet = ?"
			, "DELETE FROM Snippet WHERE snippet_id = ?"
			, "DELETE FROM CodeSnippets WHERE tool_id = ?"
			, "DELETE FROM Tools WHERE tool_id = ?"
			);
			foreach ($sql_deletes as $sql) {
				$cnn->add_param($tool_id);
				if (! $cnn->query($sql)) {
					$cnn->rollback();
					$this->session->set('message', "DELETE FAILED for {$sql}");
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "tools/tool.php?id={$tool_id}");
				}
				else {
					$cnn->reset();
				}
			}
			$this->session->set('message', "DELETE successful");
			$cnn->commit();
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "tools/");
		}
		else
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
	}

	/**
	 * Adds a new or edits an existing code snippet.
	 *
	 * If the edit succeeds or not, a redirection occurs to 
	 * the code snippet page, with errors showing or not.
	 *
	 * @return void
	 */
	public function edit_code_snippet() {
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');

		$connection= HippoDb::master();
		$tool= new Tool($connection);

		if (isset($this->handler_vars['tool_id'])) {
			$tool->tool_id= (int) $this->handler_vars['tool_id'];
			/** 
			 * These columns are ignored on update...
			 *
			 * @todo Find a more elegant way of handling this... 
			 */
			$tool->added_by= (int) $this->session->get('user_id');
			$tool->added_on= date('Y-m-d H:i:s');
		}
		else {
			$tool->fields['tool_id']->can_be_null= TRUE;
			$tool->added_by= (int) $this->session->get('user_id');
			$tool->added_on= date('Y-m-d H:i:s');
		}

		$tool->tool_type= 1; /* Code snippet */
		$tool->title= $this->handler_vars['title'];
		$tool->last_updated_by= (int) $this->session->get('user_id');
		$tool->last_updated_on= date('Y-m-d H:i:s');

		$connection->start_transaction();
		if (! $tool->save()) {
			$this->errors= $tool->errors;
			$connection->rollback();
		}
		else {
			$code_snippet= new CodeSnippet($connection);
			$code_snippet->tool_id= $tool->tool_id;
			$code_snippet->program_language= (int) $this->handler_vars['program_language'];
			$code_snippet->original_notes= $this->handler_vars['notes'];
			$code_snippet->clean_notes= TextCleaner::clean_html($this->handler_vars['notes']);
			$code_snippet->snippet_text= $this->handler_vars['snippet_text'];
			if (! $code_snippet->save()) {
				$this->errors= $code_snippet->errors;
				$connection->rollback();}
			else {
				$connection->commit();
				if (isset($this->handler_vars['tool_id']))
					HippoEvents::notify('on_after_code_snippet_edit', $this);
				else
					HippoEvents::notify('on_after_code_snippet_add', $this);
				$this->session->set('message', "Save successful");
				/* Redirect here to prevent double-saves on refresh */
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "tools/tool.php?id={$tool->tool_id}");
			}
		}
		if (isset($this->handler_vars['tool_id'])) {
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "tools/tool.php?id={$tool->tool_id}");
		}
		else
			$this->display_add_code_snippet();
	}

	/**
	 * Displays a single code snippet 
	 * @return void
	 */
	public function display_code_snippet() {
		$snippet_id= (int) $this->handler_vars['id'];
		$reader= new CodeSnippetReader(HippoDb::slave());
		$reader->tool_id= $snippet_id;
		if (! $snippet= $reader->get_one())
			$snippet= array();

		/* Ensure we override the snippet values with posted ones */
		foreach ($this->handler_vars as $key=>$value)
			if (isset($snippet[$key]))
				$snippet[$key]= $value;

		if (! empty($snippet)) {
			/* Highlight the code sample */
			$snippet['source_code']= TextDecorator::syntax_highlight($snippet['snippet_text'], $snippet['program_language_display']);

			/* 
			 * Set the editable flag if the current session 
			 * user is the one who added the snippet
			 */
			if ($snippet['added_by'] == $this->session->get('user_id')) {
				$snippet['editable_by_user']= TRUE;

				/* 
				 * Grab the various lookup value lists for the 
				 * edit Project tab and add into the template
				 */
				$reader= new ProgramLanguageReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['program_languages']= $reader->get_hash();

				/* For handling TAB within text fields and various input stuff... */
				$this->handler_vars['scripts'][]= "input-utility.js";
			}

			/* Grab the number of observing (tracking) users */
			$reader= new ObserverReader(HippoDb::slave());
			$reader->item_type= 2; /* TOOL */
			$reader->item_id = $snippet_id;
			$snippet['num_observers']= $reader->get_count();

			if (FALSE !== $this->session->get('user_id')) {
				$reader->forge_user= (int) $this->session->get('user_id');
				$snippet['observing']= ($reader->get_count() > 0);
			}
			else
				$snippet['observing']= FALSE;

			/* Grab the tags related to this snippet */
			$reader= new ItemTagReader(HippoDb::slave());
			$reader->item_type= 2; /* 2 == Tool */
			$reader->item_id= $snippet_id;
			if (! $snippet_tags= $reader->get())
				$snippet_tags= array();
			$snippet['tags']= $snippet_tags;
			$this->handler_vars['total_tags']= count($snippet_tags);
			$snippet['taglink']= AppConfig::get_config('DIR_HTTP') . 'tools/search.php?t=tag&k=';

			/* Grab the comments for the snippet too...*/
			$reader= new CommentReader(HippoDb::slave());
			$reader->item_type= 2; /* TOOL */
			$reader->item_id= $snippet_id;
			$reader->order('added_on', 'desc');
			if (! $comments= $reader->get())
				$comments= array();
			$num_comments= count($comments);
			$this->handler_vars['total_comments']= $num_comments;
			/* Convert Bug and WL# to URLs and Codify any code blocks in comments */
			for ($i=0;$i<$num_comments;++$i)
				$comments[$i]['comment_text']= TextDecorator::convert_bugs_and_worklogs($comments[$i]['clean_text']);
			$snippet['comments']= $comments;


			/* Grab the rating information for this task */
			$reader= new VoteSummaryReader(HippoDb::slave());
			$reader->item_id= $snippet_id;
			$reader->item_type= 2; /* 2 == Tool */
			if (! $snippet_ratings= $reader->get_one())
				$snippet_ratings= array();
			else
				$snippet_ratings['score']= number_format((int) $snippet_ratings['total_vote_value'] / (int) $snippet_ratings['total_votes'], 2);
			$snippet['ratings']= $snippet_ratings;

			if (FALSE !== $this->session->get('user_id')) {
				$reader= new VoteReader(HippoDb::slave());
				$reader->item_type= 2; /* 2 == Tool */
				$reader->item_id= $snippet_id;
				$reader->forge_user= $this->session->get('user_id');
				if (! $vote= $reader->get_one())
					$vote= array();
			}
			else
				$vote= array();
			$snippet['vote']= $vote;

			$rating_units= 5;
			$rating_image_width= 30;

			$this->handler_vars['rating_image_width']= $rating_image_width;
			$this->handler_vars['rating_units']= range(1, $rating_units);
			$this->handler_vars['rating_width']= $rating_units * $rating_image_width;
			$this->handler_vars['rating_unit_count']= $rating_units;
			if (!empty($snippet_ratings))
				$this->handler_vars['current_vote_width']= $rating_image_width * $snippet_ratings['score'];

			$this->handler_vars['page_title']= 'Tools :: Code Snippet :: ' . htmlentities($snippet['title']);
			$this->handler_vars['breadcrumbs'][]= array('link'=>'snippets/', 'title'=>'home');
			$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>htmlentities($snippet['title']));

			/* We need to add some javascript files and CSS files for tab panel display */
			$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/yui-utilities.js";
			$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/ext-yui-adapter.js";

			/* If the user is logged in, pull in the AJAX voting scripts */
			if (FALSE !== $this->session->get('user_id')) {
				$this->handler_vars['scripts'][]= "yui-ext/adapter/prototype/prototype.js";
				$this->handler_vars['scripts'][]= "vote.js";
			}
			$this->handler_vars['scripts'][]= "yui-ext/ext-core.js";
			$this->handler_vars['scripts'][]= "yui-ext/package/util.js";
			$this->handler_vars['scripts'][]= "yui-ext/package/tabs/tabs.js";

			$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ext-all.css";
			$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ytheme-gray.css";

		}
		$this->handler_vars['snippet']= $snippet;

		$this->display('snippet');
	}

	/**
	 * Displays the home page dashboard of the tool repository
	 * @return void
	 * @todo Make more dynamic? 
	 */

	public function display_dashboard() {
		/* Grab a little info on the last 10 modified tools */
		$reader= new ToolReader(HippoDb::slave());
		$reader->order('added_on', 'desc');
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT 
t.tool_id
, t.title
, t.added_on
, t.added_by
, ab.display_name as added_by_display
FROM Tools t
INNER JOIN ForgeUser ab
		ON t.added_by = ab.user_id
ENDOFSQL;
		if (! $newest_tools= $reader->get(10))
			$newest_tools= array();
		$this->handler_vars['total_tools']= $reader->get_count();
		$this->handler_vars['newest_tools']= $newest_tools;

		/* Grab a little info on the top 10 rated tools */
		$reader= new ToolReader(HippoDb::slave());
		/*
		 * Override default SELECT as we don't need all the info for the 
		 * dashboard and we want the ratings information too...
		 */
		$reader->select_sql =<<<ENDOFSQL
SELECT
	t.tool_id
, t.title
, t.added_by
, ab.display_name as added_by_display
, vs.total_votes
, FORMAT(vs.total_vote_value / vs.total_votes, 2) as rating
FROM Tools t
	INNER JOIN VoteSummary vs 
		ON t.tool_id = vs.item_id 
		AND vs.item_type = 2
	INNER JOIN ForgeUser ab
		ON t.added_by = ab.user_id
ENDOFSQL;
		$reader->order('rating', 'desc');
		$reader->order('total_votes', 'desc');
		if (! $top_tools= $reader->get(10))
			$top_tools= array();
		$num_tools= count($top_tools);
		for ($i=0;$i<$num_tools;++$i)
			$top_tools[$i]['rating_width']= (16 * $top_tools[$i]['rating']);
		$this->handler_vars['top_tools']= $top_tools;

		/* Grab the tags related to any tool */
		$reader= new ItemTagReader(HippoDb::slave());
		$reader->item_type= 2; /* 2 == Tool */
		$reader->order('tag_text');
		if (! $tool_tags= $reader->get())
			$tools_tags= array();
		$this->handler_vars['tool_tags']= $tool_tags;
		$this->handler_vars['tag_link']= AppConfig::get_config('DIR_HTTP') . 'tools/search.php?t=tag&k=';

		/* We need to add some javascript files for the accordian display */
		$this->handler_vars['scripts'][]= "scriptaculous/lib/prototype.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/effects.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/accordion.js";

		$this->handler_vars['page_title']= 'Tools :: Home';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'home');

		$this->display('dashboard');
	}

	/**
	 * Displays the search form or results of search
	 * @return void
	 */
	public function display_search() {
		$this->init_search_filters();
		$search_request= new ToolSearchRequest();
		$search_request->records_per_page= 20;

		if (!empty($this->handler_vars['type']))
			$search_request->reader->tool_type= (int) $this->handler_vars['type'];

		if (!empty($this->handler_vars['k'])) { /* Keywords... */
			$keywords= $this->handler_vars['k'];
			$look_in= $this->handler_vars['t'];
			if (! in_array($look_in, array('tds','tag')))
				$look_in= 'tds';
			switch ($look_in) {
			case 'tds': /* title and description... */
				$keywords= explode(',', $keywords);
				foreach ($keywords as $keyword)
					$search_request->reader->title->like("%{$keyword}%"); /* FALSE== OR condition, not AND */
				break;
			case 'tag':
				$search_request->with_tags($keywords);
				break;
			}
		}

		if (! empty($this->handler_vars['ab']))
			$search_request->reader->added_by= (int) $this->handler_vars['ab'];

		if (! $search_results= $search_request->get())
			$search_results= array();
		$num_results= count($search_results);
		for ($i=0;$i<$num_results;++$i)
			$search_results[$i]['rating_width']= (16 * $search_results[$i]['rating']);

		$this->handler_vars['search_results']= $search_results;
		$this->handler_vars['search_request']= $search_request;

		$this->handler_vars['page_title']= 'Tools :: Search Results';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'tools/', 'title'=>'tools');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'search results');

		$this->display('search-results');
	}

	/**
	 * Displays the form for contributing a new code snippet 
	 * @return void
	 */
	public function display_add_code_snippet() {
		/* 
		 * Grab the various lookup value lists for the 
		 * edit Project tab and add into the template
		 */
		$reader= new ProgramLanguageReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['program_languages']= $reader->get_hash();

		$snippet= new CodeSnippet(HippoDb::master());
		$snippet->fill_field_values($this->handler_vars);

		$this->handler_vars['page_title']= 'Tools :: Contribute a New Code Snippet';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'tools/', 'title'=>'dashboard');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'Contribute a New Code Snippet');
		$this->handler_vars['snippet']= $snippet;

		/* For handling TAB within text fields and various input stuff... */
		$this->handler_vars['scripts'][]= "input-utility.js";

		$this->display('add-snippet');
	}

	/**
	 * Sets up the dropdowns for the search results filters
	 *
	 * @return void
	 */
	private function init_search_filters() {
		/* Grab the dropdown box options */
		$reader= new ToolTypeReader(HippoDb::slave(), $this->cache);
		if (! $tool_types= $reader->get())
			$tool_types= array();
		$tmp= array('< ALL >');
		$tool_types= $tmp + hippo_otoa($tool_types);
		$this->handler_vars['tool_types']= $tool_types;

		$reader= new ForgeUserReader(HippoDb::slave(), $this->cache);
		$reader->select_sql=<<<ENDOFSQL
SELECT fu.user_id, fu.display_name
FROM ForgeUser fu
INNER JOIN Tools t
ON fu.user_id = t.added_by
ENDOFSQL;
		$reader->order('display_name');
		if (! $tool_authors= $reader->get())
			$tool_authors= array();
		$tool_authors= $tmp + hippo_otoa($tool_authors);
		$this->handler_vars['tool_authors']= $tool_authors;

		$look_ins = array();
		$look_ins['tds'] = 'Title and Description';
		$look_ins['tag'] = 'Tags';

		$this->handler_vars['look_ins']= $look_ins;

	}
}
?>
