<?php
/**
 * Module file for the person directory application
 */
class PeopleModule extends ForgeModule {
	/**
	 * Initializes the module.  Use this method to add
	 * event listeners, append to ForgeUrl, etc...
	 */
	public function init() {

		/* Add any rewriters */
		$rules= array();
		$rule= new ForgeUrlRule('people_dashboard');
		$rule->set_inbound_match("people")->set_inbound_callback(array($this, 'view_dashboard'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_add');
		$rule->set_inbound_match("people/add")->set_inbound_callback(array($this, 'add_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_delete');
		$rule->set_inbound_match("people/delete")->set_inbound_callback(array($this, 'delete_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_edit');
		$rule->set_inbound_match("people/edit")->set_inbound_callback(array($this, 'edit_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_logout');
		$rule->set_inbound_match("people/logout")
			->set_inbound_match("logout")
			->set_inbound_callback(array($this, 'logout'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_login');
		$rule->set_inbound_match("people/login")
			->set_inbound_match("login")
			->set_inbound_callback(array($this, 'login'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_view_by_id');
		$rule->set_inbound_match("/^people\/view\/(\d+)$/i")
			->set_inbound_match_vars(array('user_id'))
			->set_inbound_callback(array($this, 'view_person_by_id'))
			->set_outbound_expression('people/view/{$user_id}');
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('people_view_by_display_name');
		$rule->set_inbound_match("/^people\/view\/([[:alpha:]]+)$/i")
			->set_inbound_match("/^~([[:alpha:]]+)$/i") /* tilde common naming convention */
			->set_inbound_match_vars(array('display_name'))
			->set_inbound_callback(array($this, 'view_person_by_display_name'))
			->set_outbound_expression('people/view/{$display_name}');
		$rules[$rule->key]= $rule;

		ForgeUrl::append_rules($rules);

		HippoEvents::listen('on_before_display_template', array($this, 'add_menu_items'));
		HippoEvents::listen('on_before_display_dashboard', array($this, 'add_home_dashboard_items'));

	}

	/**
	 * Adds menu items for display in the layout template
	 */
	public function add_menu_items() {

		$controller= $this->controller;

		$main_menu_items= array();
		$main_menu_items[]= array(
			'title'=>ForgeTranslate::get('people')
			, 'link'=>ForgeUrl::out('people_dashboard')
		);

		/* Special main menu items for logged in users */
		if ($controller->user_logged_in) {
			$main_menu_items[]= array(
				'title'=>ForgeTranslate::get('edit my profile')
				, 'link'=>ForgeUrl::out('people_edit', array('tab'=>'profile','id'=>$controller->session->user_id))
				, 'special'=>TRUE
			);
			$main_menu_items[]= array(
				'title'=>ForgeTranslate::get('log out')
				, 'link'=>ForgeUrl::out('people_logout')
				, 'special'=>TRUE
			);
			$main_menu_items[]= array(
				'title'=>ForgeTranslate::get('logged in as %s', $controller->session->display_name)
				, 'link'=>''
				, 'special'=>TRUE
			);
		}
		else {
			$main_menu_items[]= array(
				'title'=>ForgeTranslate::get('log in')
				, 'link'=>ForgeUrl::out('people_login')
				, 'special'=>TRUE
			);
			$main_menu_items[]= array(
				'title'=>ForgeTranslate::get('create an account')
				, 'link'=>ForgeUrl::out('people_add')
				, 'special'=>TRUE
			);
		}

		/* Add some submenu items if we're actually in this module... */
		if ($controller->active_module == __CLASS__) {
			$submenu_items= array();
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('home')
				, 'link'=>ForgeUrl::out('people_dashboard')
			);
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('browse people')
				, 'link'=>ForgeUrl::out('people_search')
			);
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('newest people')
				, 'link'=>ForgeUrl::out('people_search', array('sortby'=>'created_on','sortorder'=>'desc'))
			);
			$controller->handler->append('submenu_items', $submenu_items);
			$main_menu_items[0]['active']= TRUE;
		}
		$controller->handler->append('menu_items', $main_menu_items);

	}

	/**
	 * Adds newest members to the home dashboard
	 */
	public function add_home_dashboard_items() {
		$controller= $this->controller;

		/* Grab a little info on the newest few people */
		$reader= new ForgeUserReader(HippoDb::slave());
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT
	fu.user_id
, fu.display_name as title
, fu.created_on as added_on
FROM ForgeUser fu
ENDOFSQL;
		$reader->order('created_on', 'desc');
		if (! $new_items= $reader->get(ForgeConfig::get('NUM_ITEMS_DASHBOARD')))
			$new_items= array();
		else {
			$num_items= count($new_items);
			for ($i=0;$i<$num_items;++$i)
				$new_items[$i]['link']= ForgeUrl::out('people_view_by_id', array('user_id'=>$new_items[$i]['user_id']));
		}
		$controller->handler->append('new_items', $new_items);
	}

	/**
	 * Displays the main people dashboard
	 */
	public function view_dashboard() {
		$controller= $this->controller;
		$handler= $controller->handler;

		/* Grab a little info on the last 15 newest people (Forge accounts) */
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->order('created_on', 'desc');
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT 
	fu.user_id
, fu.display_name
, fu.created_on
FROM ForgeUser fu
ENDOFSQL;
		if (! $newest_people= $reader->get(10))
			$newest_people= array();
		$handler->assign('total_people', $reader->get_count());
		$handler->assign('newest_people', $newest_people);

		/* Grab a little info on the top 10 rated people */
		/*
		 * Override default SELECT as we don't need all the info for the 
		 * dashboard and we want the ratings information too...
		 */
		$cnn= HippoDb::slave();
		$sql=<<<ENDOFSQL
SELECT 
	fu.user_id
, fu.display_name
, COUNT(*) as num_tools_added
FROM ForgeUser fu
INNER JOIN Tools t
ON fu.user_id = t.added_by
GROUP BY fu.user_id
ORDER BY num_tools_added DESC
LIMIT 10
ENDOFSQL;
		if (! $active_people= $cnn->get_rows($sql))
			$active_people= array();
		$handler->assign('active_people', $active_people);

		/* Grab the tags related to any forge user */
		$reader= new ItemTagReader(HippoDb::slave());
		$reader->item_type= 4; /* 4 == Forge User */
		$reader->order('tag_text');
		if (! $user_tags= $reader->get())
			$user_tags= array();
		$handler->assign('user_tags', $user_tags);
		$handler->assign('tag_link', ForgeUrl::out('people_search_by_tag'));

		$handler->assign('page_title', ForgeTranslate::get('People') . ' :: ' . ForgeTranslate::get('Home'));

		/* We need to add some javascript files for the accordian display */
		$handler->append('scripts', "scriptaculous/lib/prototype.js");
		$handler->append('scripts', "scriptaculous/src/effects.js");
		$handler->append('scripts', "scriptaculous/src/accordion.js");

		$handler->display('people/dashboard');
	}

	/**
	 * Grabs the user's ID from their display name and
	 * calls display_person
	 */
	public function view_person_by_display_name() {
		$handler= $this->controller->handler;
		if (empty($handler->handler_vars['display_name']))
			$handler->display('404');

		$sql= "SELECT user_id FROM ForgeUser WHERE display_name = ?";
		$cnn= HippoDb::slave();
		$cnn->add_param($handler->handler_vars['display_name']);
		if (FALSE === ($user_id= $cnn->get_value($sql)))
			$handler->display('404');

		$this->display_person($user_id);
	}

	/**
	 * Merely calls display_person after verifying ID
	 */
	public function view_person_by_id() {
		$handler= $this->controller->handler;
		if (! empty($handler->handler_vars['user_id']))
			$handler->display('404');
		$this->display_person((int) $handler->handler_vars['user_id']);
	}

	/**
	 * Displays a user's profile
	 */
	private function display_person($user_id) {
		$handler= $this->controller->handler;
		$session= $this->controller->session;

		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->user_id= $user_id;
		if (! $person= $reader->get_one())
			$person= array();

		if (! empty($person)) {
			/* We need to escape output here...*/
			$person['bio']= TextCleaner::clean_html($person['bio']); /** @TODO Cache this */

			/* 
			 * Set the editable flag if the current session 
			 * user is the one who added the snippet
			 */
			if ($person['user_id'] == $session->get('user_id')) {
				$person['editable_by_user']= TRUE;

				/* 
				 * Grab the various lookup value lists for the 
				 * Edit Profile tab and add into the template
				 */
				$reader= new LanguageReader(HippoDb::slave(), $this->cache);
				$reader->order('name');
				$handler->assign('languages', $reader->get_hash());

				$reader= new TimezoneReader(HippoDb::slave(), $this->cache);
				$reader->order('utc_offset_min');
				$handler->assign('timezones', $reader->get_hash());

				$reader= new CountryReader(HippoDb::slave(), $this->cache);
				$reader->order('name');
				$handler->assign('countries', $reader->get_hash());
			}

			/* Grab the tags related to this user */
			$reader= new ItemTagReader(HippoDb::slave());
			$reader->item_type= 4; /* 4 === Forge User */
			$reader->item_id= $user_id;
			if (! $user_tags= $reader->get())
				$user_tags= array();
			$person['tags']= $user_tags;
			$handler->assign('total_tags', count($user_tags));
			$person['taglink']= ForgeConfig::get('HTTP_ROOT') . 'people/search.php?t=tag&k=';

			$handler->assign('page_title', 'People :: ' . htmlentities($person['display_name']));
		}

		/* We need to add some javascript files and CSS files for tab panel display */
		$handler->append('scripts', "yui-ext/adapter/yui/yui-utilities.js");
		$handler->append('scripts', "yui-ext/adapter/yui/ext-yui-adapter.js");
		$handler->append('scripts', "yui-ext/ext-core.js");
		$handler->append('scripts', "yui-ext/package/util.js");
		$handler->append('scripts', "yui-ext/package/tabs/tabs.js");

		$handler->append('stylesheets', "../js/yui-ext/resources/css/ext-all.css");
		$handler->append('stylesheets', "../js/yui-ext/resources/css/ytheme-gray.css");

		$handler->assign('person', $person);

		$handler->display('people/person');
	}

	/**
	 * Adds a new person to the application
	 */
	public function add_person() {
		if (! HippoHttpResponse::is_postback())
			$this->display_add_person();
	}

	/**
	 * Displays the form for registering a new forge user 
	 *
	 * @return void
	 */
	public function display_add_person() {
		$handler= $this->controller->handler;
		$cache= $this->controller->cache;

		/* 
		 * Grab the various lookup value lists for the 
		 * new account form and add into the template
		 */
		$reader= new LanguageReader(HippoDb::slave(), $cache);
		$reader->order('name');
		$handler->assign('languages', $reader->get_hash());

		$reader= new TimezoneReader(HippoDb::slave(), $cache);
		$reader->order('utc_offset_min');
		$handler->assign('timezones', $reader->get_hash());

		$reader= new CountryReader(HippoDb::slave(), $cache);
		$reader->order('name');
		$handler->assign('countries', $reader->get_hash());

		$person= new ForgeUser(HippoDb::master());
		$person->fill_field_values($handler->handler_vars);

		$person_profile= new ForgeUserProfile(HippoDb::master());
		$person_profile->fill_field_values($handler->handler_vars);

		$handler->assign('page_title', ForgeTranslate::get('People') . ' :: ' . ForgeTranslate::get('Create Account'));
		$handler->assign('person', $person);
		$handler->assign('person_profile', $person_profile);

		$handler->display('people/add');
	}

	/**
	 * Logs out the forge user and redirects to a home page
	 *
	 * @return void
	 */
	public function logout() {
		HippoEvents::notify('on_before_person_logout', $this);
		$this->controller->session->destroy();
		HippoHttpResponse::redirect(ForgeConfig::get('HTTP_ROOT'));
  }

	/**
	 * Logs a user into the application
	 *
	 * @return TRUE or FALSE
	 */
	public function login() {
		if (! HippoHttpResponse::is_postback())
			$this->display_login();

		$handler= $this->controller->handler;

		$password= null;
		$email= null;
		$url_redirect= '';
		$persist_session= "0";
		
		/* Init some local variables */
		if (isset($handler->handler_vars['persist_session']))
			$persist_session= $handler->handler_vars['persist_session'] == 'on' ? "1" : "0";
		if (isset($handler->handler_vars['password']))
			$password = $handler->handler_vars['password'];
		if (isset($handler->handler_vars['email']))
			$email= $handler->handler_vars['email'];
		if (isset($handler->handler_vars['url_redirect'])) {
			$url_redirect= $handler->handler_vars['url_redirect'];

			/* Fix for Bug #34956  	Login redirection vulnerability */
			$url_redirect_pieces= parse_url($url_redirect);
			if (isset($url_redirect_pieces['host']) &&
				! in_array($url_redirect_pieces['host'], ForgeConfig::get('VALID_REDIRECT_HOSTS')))
				$url_redirect= '';
		}
		else
			$url_redirect= '';

		/* Do some rudimentary sanity checks on post */
		if (empty($password))
			$handler->add_error('password', ForgeTranslate::get('Please type a password'));
		if (empty($email))
			$handler->add_error('email', ForgeTranslate::get('Email address is required'));
		if (FALSE === strpos($email, '@'))
			$handler->add_error('email', ForgeTranslate::get('It looks like you did not enter a valid email address.  Remember to log in with your email address, not a username'));

		if (!empty($handler->errors)) {
			/* Rudimentary errors occurred... */
			$this->display_login(); 
		}

		if (FALSE === ($user_id= $this->passes_local_authentication($email, $password))) {
			/* Failed local authentication.  Error out and retry */
			$handler->add_error('email', ForgeTranslate::get('Invalid user credentials supplied.'));
			$this->display_login();
			exit;
		}
					
		/*
		 * We get here if all the authentication checks have been passed.  If we get here, the above code
		 * has guaranteed that we have the user_id variable correctly populated
		 */

		/* Update the local user's login time */
		$user= new ForgeUser(HippoDb::master(), $user_id);
		$user->update_login_time();

		/* 
		 * Set up the session.
		 *
		 * At this point, we should have the local forge user ID, the
		 * local wiki user ID, the display name and whether or not
		 * to persist the session.		 
		 */
		$this->session->set('user_id', $user_id);
		$this->session->set('is_admin', $user->is_admin);
		$this->session->set('persist_session', $persist_session);
		$this->session->set('display_name', $user->display_name);
		$this->session->set('last_login_time', date('Y-m-d H:i:s'));

		HippoEvents::notify('on_after_person_login');
		HippoHttpResponse::redirect($url_redirect);
	}

	/**
	 * Verify credentials against a local forge user account
	 *
	 * @return	bool					Checks out?
	 * @param		email					Email address
	 * @param		password			Password to check
	 */
	private function passes_local_authentication($email, $password) {
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT password FROM ForgeUser 
ENDOFSQL;
		$reader->email= $email;
		if (FALSE === ($forge_user= $reader->get_one()))
			return FALSE;
		return ($forge_user['password'] == md5($password));
	}

	/**
	 * Displays the login form

	 * @return void
	 */
	public function display_login() {
		$handler= $this->controller->handler;

		/* Fix for Bug #34956  	Login redirection vulnerability */
		if (isset($handler->handler_vars['url_redirect']))
			$url_redirect= $handler->handler_vars['url_redirect'];
		else {
			if (isset($_SERVER['HTTP_REFERER']))
				$url_redirect= $_SERVER['HTTP_REFERER'];
			else
				$url_redirect= '';
		}

		$url_redirect_pieces= parse_url($url_redirect);
		if (isset($url_redirect_pieces['host']) &&
			! in_array($url_redirect_pieces['host'], ForgeConfig::get('VALID_REDIRECT_HOSTS')))
			$url_redirect= '';
		if ($url_redirect == '')
			$url_redirect= ForgeConfig::get('HTTP_ROOT');
		$handler->assign('url_redirect', $url_redirect);

		if (!isset($handler->handler_vars['password']))
			$handler->assign('password', '');
		if (!isset($handler->handler_vars['email']))
			$handler->assign('email', '');

		$handler->assign('page_title', ForgeTranslate::get('People') . ' :: ' . ForgeTranslate::get('Login'));

		/** @TODO Move to plugin 
		if ($show_captcha)
			$handler->assign('captcha_html', ForgeCaptcha::get_html());
		 */
		$handler->display('people/login');
	}
}
?>
