<?php

/**
 * Universal FeedGenerator class
 * 
 * (based on the Universal Feed Generator by Anis uddin Ahmad <anisniit@gmail.com>,
 * http://www.ajaxray.com/projects/rss, version 1.0)
 *
 * Generate RSS 1.0, RSS2.0 and ATOM Feed
 *                             
 * @author Robert van der Mast <robertvandermast@gmail.com>
 * @package Universal FeedGenerator
 * @version 1.1
 * @license GNU General Public License
 */

if (!defined('FEEDGENERATOR_DIR'))
	define('FEEDGENERATOR_DIR', dirname(__FILE__));

/* The following file is necessary for the FeedGenerator */
include(FEEDGENERATOR_DIR . '/FeedItem.php');

// RSS 0.90  Officially obsoleted by 1.0
// RSS 0.91, 0.92, 0.93 and 0.94  Officially obsoleted by 2.0
// So, define constants for RSS 1.0, RSS 2.0 and ATOM 	

if (!defined('RSS1'))
	define('RSS1', 'RSS 1.0', true);
if (!defined('RSS2'))
	define('RSS2', 'RSS 2.0', true);
if (!defined('ATOM'))
	define('ATOM', 'ATOM', true);
 
class FeedGenerator
{
	/* Collection of channel elements */
	private $channels      = array();
	/* Collection of items as object of FeedItem class */
	private $items         = array();
	/* Store some other version wise data */
	private $data          = array();
	/* The tag names which have to encoded as CDATA */
	private $CDATAEncoding = array();
	 
	private $protocol   = null; 
	
	/**
	 * Constructor
	 * 
	 * @param    constant    the version constant (RSS1/RSS2/ATOM).       
	 */ 
	public function __construct($protocol = RSS2) {	
		$this->protocol = $protocol;
			
		// Setting default value for assential channel elements
		$this->channels['title']	= $protocol . ' Feed';
		$this->channels['link']		= '';
		
		//Tag names to encode in CDATA
		$this->CDATAEncoding	= array('description', 'content:encoded', 'summary', 'subtitle');
	}

	/**
	 * Set a channel element
	 * @access   public
	 * @param    string  name of the channel tag
	 * @param    string  content of the channel tag
	 * @return   void
	 */
	public function setChannelElement($elementName, $content) {
		$this->channels[$elementName] = $content ;
	}
	
	/**
	 * Set multiple channel elements from an array. Array elements 
	 * should be 'channelName' => 'channelContent' format.
	 * 
	 * @access   public
	 * @param    array   array of channels
	 * @return   void
	 */
	public function setChannelElementsFromArray($elementArray) {
		if (!is_array($elementArray))
			return;
			
		foreach ($elementArray as $elementName => $content) {
			$this->setChannelElement($elementName, $content);
		}
	}
	
	/**
	 * Genarate the actual RSS/ATOM file
	 * 
	 * @access   public
	 * @return   void
	 */ 
	public function generateFeed() {
		return $this->printHead().$this->printChannels().$this->printItems().$this->printTale();
	}
	
	/**
	 * Create a new FeedItem.
	 * 
	 * @access   public
	 * @return   object  instance of FeedItem class
	 */
	public function createNewItem() {
		$Item = new FeedItem($this->protocol);
		return $Item;
	}
	
	/**
	 * Add a FeedItem to the main class
	 * 
	 * @access   public
	 * @param    object  instance of FeedItem class
	 * @return   void
	 */
	public function addItem($feedItem) {
		$this->items[] = $feedItem;    
	}
	
	/**
	 * Set the 'title' channel element
	 * 
	 * @access   public
	 * @param    string  value of 'title' channel tag
	 * @return   void
	 */
	public function setTitle($title) {
		$this->setChannelElement('title', $title);
	}
	
	/**
	 * Set the 'description' channel element
	 * 
	 * @access   public
	 * @param    string  value of 'description' channel tag
	 * @return   void
	 */
	public function setDescription($desciption) {
		if ($this->protocol == ATOM) {
			$this->setChannelElement('subtitle', $desciption);	
		}
		else {
			$this->setChannelElement('description', $desciption);
		}
	}
	
	/**
	 * Set the 'link' channel element
	 * 
	 * @access   public
	 * @param    string  value of 'link' channel tag
	 * @return   void
	 */
	public function setLink($link) {
		$this->setChannelElement('link', $link);
	}
	
	/**
	 * Set the 'image' channel element
	 * 
	 * @access   public
	 * @param    string  title of image
	 * @param    string  link url of the imahe
	 * @param    string  path url of the image
	 * @return   void
	 */
	public function setImage($title, $link, $url) {
		$this->setChannelElement('image', array('title'=>$title, 'link'=>$link, 'url'=>$url));
	}
	
	/**
	 * Set the 'about' channel element. Only for RSS 1.0
	 * 
	 * @access   public
	 * @param    string  value of 'about' channel tag
	 * @return   void
	 */
	public function setChannelAbout($url) {
		$this->data['ChannelAbout'] = $url;    
	}
	
	/**
	 * Genarates an UUID
	 * @author     Anis uddin Ahmad <admin@ajaxray.com>
	 * @param      string  an optional prefix
	 * @return     string  the formated uuid
	 */
	
	public static function uuid($key = null, $prefix = '') {
		$key = ($key == null)? uniqid(rand()) : $key;
		$chars = md5($key);
		$uuid  = substr($chars,0,8) . '-';
		$uuid .= substr($chars,8,4) . '-';
		$uuid .= substr($chars,12,4) . '-';
		$uuid .= substr($chars,16,4) . '-';
		$uuid .= substr($chars,20,12);

		return $prefix . $uuid;
	}
		
	/**
	 * Prints the xml and rss namespace
	 * 
	 * @access   private
	 * @return   void
	 */
	private function printHead() {
		$out  = '<?xml version="1.0" encoding="utf-8"?>' . "\n";
		
		if ($this->protocol == RSS2) {
			$out .= '<rss version="2.0"
			 		xmlns:content="http://purl.org/rss/1.0/modules/content/"
					xmlns:wfw="http://wellformedweb.org/CommentAPI/"
				  >' . PHP_EOL;
		}    
		elseif ($this->protocol == RSS1) {
			$out .= '<rdf:RDF 
					 xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
					 xmlns="http://purl.org/rss/1.0/"
					 xmlns:dc="http://purl.org/dc/elements/1.1/"
					>' . PHP_EOL;;
		}
		else if ($this->protocol == ATOM) {
			$out .= '<feed xmlns="http://www.w3.org/2005/Atom">' . PHP_EOL;;
		}
		
		return $out;
	}
	
	/**
	 * Closes the open tags at the end of file
	 * 
	 * @access   private
	 * @return   void
	 */
	private function printTale() {
		if ($this->protocol == RSS2) {
			return '</channel>' . PHP_EOL . '</rss>'; 
		}    
		elseif ($this->protocol == RSS1) {
			return '</rdf:RDF>';
		}
		else if ($this->protocol == ATOM) {
			return '</feed>';
		}	  
	}

	/**
	 * Creates a single node as xml format
	 * 
	 * @access   private
	 * @param    string  name of the tag
	 * @param    mixed   tag value as string or array of nested tags in 'tagName' => 'tagValue' format
	 * @param    array   Attributes(if any) in 'attrName' => 'attrValue' format
	 * @return   string  formatted xml tag
	 */
	private function makeNode($tagName, $tagContent, $attributes = null) {        
		$nodeText = '';
		$attrText = '';

		if (is_array($attributes)) {
			foreach ($attributes as $key => $value) {
				$attrText .= " $key=\"$value\" ";
			}
		}
		
		if (is_array($tagContent) && $this->protocol == RSS1) {
			$attrText = ' rdf:parseType="Resource"';
		}
				
		$attrText .= (in_array($tagName, $this->CDATAEncoding) && $this->protocol == ATOM)? ' type="html"' : '';
		$nodeText .= (in_array($tagName, $this->CDATAEncoding))? "<{$tagName}{$attrText}><![CDATA[" : "<{$tagName}{$attrText}>";
		 
		if (is_array($tagContent)) { 
			foreach ($tagContent as $key => $value) {
				$nodeText .= $this->makeNode($key, $value);
			}
		}
		else {
			$nodeText .= (in_array($tagName, $this->CDATAEncoding))? $tagContent : htmlentities($tagContent);
		}           
			
		$nodeText .= (in_array($tagName, $this->CDATAEncoding))? "]]></$tagName>" : "</$tagName>";

		return $nodeText . PHP_EOL;
	}
	
	/**
	 * @desc     Print channels
	 * @access   private
	 * @return   void
	 */
	private function printChannels() {
		$print_channels= '';
		//Start channel tag
		switch ($this->protocol) {
		   case RSS2: 
				$print_channels.= '<channel>' . PHP_EOL;        
				break;
		   case RSS1: 
				$print_channels.= (isset($this->data['ChannelAbout']))? "<channel rdf:about=\"{$this->data['ChannelAbout']}\">" : "<channel rdf:about=\"{$this->channels['link']}\">";
				break;
		}
		
		//Print Items of channel
		foreach ($this->channels as $key => $value) {
			if ($this->protocol == ATOM && $key == 'link') {
				//ATOM prints link element as href attribute
				$print_channels.= $this->makeNode($key, '', array('href' => $value, 'rel' => 'self'));
				//Add the id for ATOM
				$print_channels.= $this->makeNode('id', $this->uuid($value,'urn:uuid:'));
			}
			else {
				$print_channels.= $this->makeNode($key, $value);
			}    
			
		}
		
		//RSS 1.0 have special tag <rdf:Seq> with channel 
		if ($this->protocol == RSS1) {
			$print_channels.= "<items>" . PHP_EOL . "<rdf:Seq>" . PHP_EOL;
			foreach ($this->items as $item) {
				$thisItems = $item->getElements();
				$print_channels.= "<rdf:li resource=\"{$thisItems['link']['content']}\"/>" . PHP_EOL;
			}
			$print_channels.= "</rdf:Seq>" . PHP_EOL . "</items>" . PHP_EOL . "</channel>" . PHP_EOL;
		}
		
		return $print_channels;
	}
	
	/**
	 * Prints formatted feed items
	 * 
	 * @access   private
	 * @return   void
	 */
	private function printItems() {    
		$print_items= '';
		
		foreach ($this->items as $item) {
			$thisItems = $item->getElements();
			
			//the argument is printed as rdf:about attribute of item in rss 1.0 
			$print_items.=  $this->startItem($thisItems['link']['content']);
			
			foreach ($thisItems as $feedItem) {
				$print_items.= $this->makeNode($feedItem['name'], $feedItem['content'], $feedItem['attributes']); 
			}
			$print_items.= $this->endItem();
		}
		
		return $print_items;
	}
	
	/**
	 * Make the starting tag of channels
	 * 
	 * @access   private
	 * @param    string  The vale of about tag which is used for only RSS 1.0
	 * @return   void
	 */
	private function startItem($about = false) {
		if ($this->protocol == RSS2) {
			return '<item>' . PHP_EOL; 
		}    
		elseif ($this->protocol == RSS1) {
			if ($about) {
				return "<item rdf:about=\"$about\">" . PHP_EOL;
			}
			else {
				die('link element is not set .\n It\'s required for RSS 1.0 to be used as about attribute of item');
			}
		}
		else if ($this->protocol == ATOM) {
			return "<entry>" . PHP_EOL;
		}
	}
	
	/**
	 * Closes feed item tag
	 * 
	 * @access   private
	 * @return   void
	 */
	private function endItem() {
		if ($this->protocol == RSS2 || $this->protocol == RSS1) {
			return '</item>' . PHP_EOL; 
		}    
		else if ($this->protocol == ATOM) {
			return "</entry>" . PHP_EOL;
		}
	}
}
 
?>