<?php

/**
 * Universal FeedGenerator class
 * 
 * (based on the Universal Feed Generator by Anis uddin Ahmad <anisniit@gmail.com>,
 * http://www.ajaxray.com/projects/rss, version 1.0)
 *
 * FeedItem class - Used as feed element in FeedGenerator class
 *                             
 * @author Robert van der Mast <robertvandermast@gmail.com>
 * @package Universal FeedGenerator
 * @version 1.1
 * @license GNU General Public License
 */
 
class FeedItem
{
	private $elements = array();    //Collection of feed elements
	private $protocol;
	
	/**
	 * Constructor 
	 * 
	 * @param    contant     (RSS1/RSS2/ATOM) RSS2 is default. 
	 */ 
	public function __construct($protocol = RSS2) {    
		$this->protocol = $protocol;
	}
	
	/**
	 * Add an element to elements array
	 * 
	 * @access   public
	 * @param    srting  The tag name of an element
	 * @param    srting  The content of tag
	 * @param    array   Attributes(if any) in 'attrName' => 'attrValue' format
	 * @return   void
	 */
	public function addElement($elementName, $content, $attributes = null) {
		$this->elements[$elementName]['name']       = $elementName;
		$this->elements[$elementName]['content']    = $content;
		$this->elements[$elementName]['attributes'] = $attributes;
	}
	
	/**
	 * Set multiple feed elements from an array. 
	 * Elements which have attributes cannot be added by this method
	 * 
	 * @access   public
	 * @param    array   array of elements in 'tagName' => 'tagContent' format.
	 * @return   void
	 */
	public function addElementArray($elementArray) {
		if (!is_array($elementArray))
			return;
			
		foreach ($elementArray as $elementName => $content) {
			$this->addElement($elementName, $content);
		}
	}
	
	/**
	 * Return the collection of elements in this feed item
	 * 
	 * @access   public
	 * @return   array
	 */
	public function getElements() {
		return $this->elements;
	}
	
	// Wrapper functions ------------------------------------------------------
	
	/**
	 * Set the 'dscription' element of feed item
	 * 
	 * @access   public
	 * @param    string  The content of 'description' element
	 * @return   void
	 */
	public function setDescription($description) {
		$tag = ($this->protocol == ATOM)? 'summary' : 'description'; 
		$this->addElement($tag, $description);
	}
	
	/**
	 * @desc     Set the 'title' element of feed item
	 * @access   public
	 * @param    string  The content of 'title' element
	 * @return   void
	 */
	public function setTitle($title) {
		$this->addElement('title', $title);  	
	}
	
	/**
	 * Set the 'date' element of feed item
	 * 
	 * @access   public
	 * @param    string  The content of 'date' element
	 * @return   void
	 */
	public function setDate($date) 
	{
		if (! is_numeric($date)) {
			$date = strtotime($date);
		}
		
		if ($this->protocol == ATOM) {
			$tag    = 'updated';
			$value  = date(DATE_ATOM, $date);
		}        
		elseif ($this->protocol == RSS2) {
			$tag    = 'pubDate';
			$value  = date(DATE_RSS, $date);
		}
		else {
			$tag    = 'dc:date';
			$value  = date("Y-m-d", $date);
		}
		
		$this->addElement($tag, $value);    
	}
	
	/**
	 * Set the 'link' element of feed item
	 * 
	 * @access   public
	 * @param    string  The content of 'link' element
	 * @return   void
	 */
	public function setLink($link) {
		if ($this->protocol == RSS2 || $this->protocol == RSS1) {
			$this->addElement('link', $link);
		}
		else {
			$this->addElement('link','',array('href'=>$link));
			$this->addElement('id', FeedGenerator::uuid($link,'urn:uuid:'));
		} 
		
	}
	
	/**
	 * Set the 'encloser' element of feed item
	 * For RSS 2.0 only
	 * 
	 * @access   public
	 * @param    string  The url attribute of encloser tag
	 * @param    string  The length attribute of encloser tag
	 * @param    string  The type attribute of encloser tag
	 * @return   void
	 */
	public function setEncloser($url, $length, $type) {
		$attributes = array('url'=>$url, 'length'=>$length, 'type'=>$type);
		$this->addElement('enclosure','',$attributes);
	}
	
}

?>